@@ -1,21 +1,29 @@
 #include <boost/foreach.hpp>
+#include <boost/filesystem.hpp>
 #include <boost/regex.hpp>
 #include <sstream>
 #include <iostream>
 #include <locale.h>
 
 #include "cgalutils.h"
+#include "memory.h"
 #include "export.h"
 #include "polyset.h"
 #include "CGAL_Nef_polyhedron.h"
+#pragma push_macro("NDEBUG")
+#undef NDEBUG
+#include <CGAL/IO/Nef_polyhedron_iostream_3.h>
+#pragma pop_macro("NDEBUG")
 
 using namespace CGALUtils;
+namespace fs = boost::filesystem;
 
 // Nef polyhedron are using CGAL_Kernel3 (Cartesian<Gmpq>)
 // Triangulation will use Epick
 typedef CGAL::Epick K;
 typedef CGAL::Polyhedron_3<K> PolyhedronK;
 
+#include <boost/algorithm/string.hpp>
 #include <boost/assign/std/vector.hpp>
 #include <boost/assign/list_of.hpp>
 using namespace boost::assign; // bring 'operator+=()' into scope
@@ -170,7 +178,11 @@ std::vector<Color4f> colors = boost::assign::list_of
   (Color4f(154, 205, 50));
 
 #include <boost/unordered_set.hpp>
+#pragma push_macro("NDEBUG")
+#undef NDEBUG
 #include <CGAL/convex_hull_3.h>
+#pragma pop_macro("NDEBUG")
+
 template<typename Polyhedron>
 bool is_weakly_convex(Polyhedron const& p) {
   for (typename Polyhedron::Edge_const_iterator i = p.edges_begin(); i != p.edges_end(); ++i) {
@@ -273,13 +285,13 @@ void decompose(const CGAL_Nef_polyhedron3 *N, Output out_iter)
   }
 }
 
-Geometry const * minkowskitest(const Geometry::ChildList &children)
+Geometry const * minkowskitest(const Geometry::Geometries &children)
 {
   CGAL::Timer t,t_tot;
   assert(children.size() >= 2);
   // Iterate over children, perform pairwise minkowski on children:
   //   operands = [ch, ch+1]
-  Geometry::ChildList::const_iterator minkowski_ch_it = children.begin();
+  Geometry::Geometries::const_iterator minkowski_ch_it = children.begin();
   t_tot.start();
   Geometry const *operands[2] = {minkowski_ch_it->second.get(), NULL};
   try {
@@ -417,10 +429,10 @@ Geometry const * minkowskitest(const Geometry::ChildList &children)
       } else if (!result_parts.empty()) {
         t.start();
         PRINTDB("Minkowski: Computing union of %d parts",result_parts.size());
-        Geometry::ChildList fake_children;
-        for (std::list<PolyhedronK>::iterator i = result_parts.begin(); i != result_parts.end(); ++i) {
+        Geometry::Geometries fake_children;
+        for (const auto &polyhedron : result_parts) {
           PolySet ps(3,true);
-          createPolySetFromPolyhedron(*i, ps);
+          createPolySetFromPolyhedron(polyhedron, ps);
           fake_children.push_back(std::make_pair((const AbstractNode*)NULL,
                                                  shared_ptr<const Geometry>(createNefPolyhedronFromGeometry(ps))));
         }
@@ -629,21 +641,30 @@ int main(int argc, char *argv[])
   OpenSCAD::debug = "decompose";
 
   PolySet *ps = NULL;
+  CGAL_Nef_polyhedron *N = NULL;
   if (argc == 2) {
-    if (!(ps = import_stl(argv[1]))) {
-      std::cerr << "Error importing STL " << argv[1] << std::endl;
-      exit(1);
+    std::string filename(argv[1]);
+    std::string suffix = fs::path(filename).extension().generic_string();
+    if (suffix == ".stl") {
+      if (!(ps = import_stl(filename))) {
+        std::cerr << "Error importing STL " << filename << std::endl;
+        exit(1);
+      }
+      std::cerr << "Imported " << ps->numPolygons() << " polygons" << std::endl;
+    }
+    else if (suffix == ".nef3") {
+      N = new CGAL_Nef_polyhedron(new CGAL_Nef_polyhedron3);
+      std::ifstream stream(filename.c_str());
+      stream >> *N->p3;
+      std::cerr << "Imported Nef polyhedron" << std::endl;
     }
-    std::cerr << "Imported " << ps->numPolygons() << " polygons" << std::endl;
   }
   else {
     std::cerr << "Usage: " << argv[0] << " <file.stl> <file.stl>" << std::endl;
     exit(1);
   }
 
-  Geometry::ChildList children;
-
-  CGAL_Nef_polyhedron *N = createNefPolyhedronFromGeometry(*ps);
+  if (ps && !N) N = createNefPolyhedronFromGeometry(*ps);
 
   std::vector<PolyhedronK> result;
   decompose(N->p3.get(), std::back_inserter(result));
@@ -652,14 +673,14 @@ int main(int argc, char *argv[])
 
   int idx = 0;
   BOOST_FOREACH(const PolyhedronK &P, result) {
-    PolySet result_ps(3);
-    if (CGALUtils::createPolySetFromPolyhedron(P, result_ps)) {
+    PolySet *result_ps = new PolySet(3);
+    if (CGALUtils::createPolySetFromPolyhedron(P, *result_ps)) {
       std::cerr << "Error converting to PolySet\n";
     }
     else {
       std::stringstream ss;
       ss << "out" << idx++ << ".stl";
-      exportFileByName(&result_ps, OPENSCAD_STL, ss.str().c_str(), ss.str().c_str());
+      exportFileByName(shared_ptr<const Geometry>(result_ps), OPENSCAD_STL, ss.str().c_str(), ss.str().c_str());
       std::cout << "color([" << colors[idx%147][0] << "," << colors[idx%147][1] << "," << colors[idx%147][2] << "]) " << "import(\"" << ss.str() << "\");\n";
     }
   }
