@@ -9,18 +9,18 @@
 class CGAL_Nef_polyhedron : public Geometry
 {
 public:
-	CGAL_Nef_polyhedron(CGAL_Nef_polyhedron3 *p = NULL);
+	CGAL_Nef_polyhedron(CGAL_Nef_polyhedron3 *p = nullptr);
 	CGAL_Nef_polyhedron(const CGAL_Nef_polyhedron &src);
 	~CGAL_Nef_polyhedron() {}
 
-	virtual size_t memsize() const;
+	size_t memsize() const override;
 	// FIXME: Implement, but we probably want a high-resolution BBox..
-	virtual BoundingBox getBoundingBox() const { assert(false && "not implemented"); }
-	virtual std::string dump() const;
-	virtual unsigned int getDimension() const { return 3; }
+	BoundingBox getBoundingBox() const override { assert(false && "not implemented"); return BoundingBox(); }
+	std::string dump() const override;
+	unsigned int getDimension() const override { return 3; }
   // Empty means it is a geometric node which has zero area/volume
-	virtual bool isEmpty() const;
-	virtual Geometry *copy() const { return new CGAL_Nef_polyhedron(*this); }
+	bool isEmpty() const override;
+	Geometry *copy() const override { return new CGAL_Nef_polyhedron(*this); }
 
 	void reset() { p3.reset(); }
 	CGAL_Nef_polyhedron &operator+=(const CGAL_Nef_polyhedron &other);
@@ -30,7 +30,7 @@ public:
 // FIXME: Deprecated by CGALUtils::createPolySetFromNefPolyhedron3
 //	class PolySet *convertToPolyset() const;
 	void transform( const Transform3d &matrix );
-	void resize(Vector3d newsize, const Eigen::Matrix<bool,3,1> &autosize);
+	void resize(const Vector3d &newsize, const Eigen::Matrix<bool,3,1> &autosize);
 
 	shared_ptr<CGAL_Nef_polyhedron3> p3;
 };
