@@ -10,8 +10,8 @@
 class Geometry
 {
 public:
-  typedef std::pair<const class AbstractNode *, shared_ptr<const Geometry> > ChildItem;
-  typedef std::list<ChildItem> ChildList;
+  typedef std::pair<const class AbstractNode *, shared_ptr<const Geometry>> GeometryItem;
+  typedef std::list<GeometryItem> Geometries;
 
 	Geometry() : convexity(1) {}
 	virtual ~Geometry() {}
