@@ -22,18 +22,18 @@ struct IndexedTriangleMesh {
 // Indexed polygon mesh, where each polygon can have holes
 struct IndexedPolyMesh {
 	std::vector<Vector3f> vertices;
-	std::vector<std::vector<IndexedFace> > polygons;
+	std::vector<std::vector<IndexedFace>> polygons;
 };
 
 namespace GeometryUtils {
 	bool tessellatePolygon(const Polygon &polygon,
 												 Polygons &triangles,
-												 const Vector3f *normal = NULL);
+												 const Vector3f *normal = nullptr);
 	bool tessellatePolygonWithHoles(const Vector3f *vertices,
 																	const std::vector<IndexedFace> &faces, 
 																	std::vector<IndexedTriangle> &triangles,
-																	const Vector3f *normal = NULL);
+																	const Vector3f *normal = nullptr);
 
-	int findUnconnectedEdges(const std::vector<std::vector<IndexedFace> > &polygons);
+	int findUnconnectedEdges(const std::vector<std::vector<IndexedFace>> &polygons);
 	int findUnconnectedEdges(const std::vector<IndexedTriangle> &triangles);
 }
