@@ -6,53 +6,49 @@
 #include "ui_MainWindow.h"
 #include "UIUtils.h"
 #include "openscad.h"
-#include "modcontext.h"
+#include "builtincontext.h"
 #include "module.h"
+#include "ModuleInstantiation.h"
 #include "Tree.h"
 #include "memory.h"
 #include "editor.h"
+#include "export.h"
 #include <vector>
 #include <QMutex>
-#include <QSet>
+#include <QElapsedTimer>
 #include <QTime>
+#include <QIODevice>
+#include "input/InputDriver.h"
 
-enum export_type_e {
-	EXPORT_TYPE_UNKNOWN,
-	EXPORT_TYPE_STL,
-	EXPORT_TYPE_AMF,
-	EXPORT_TYPE_OFF
-};
-
-class MainWindow : public QMainWindow, public Ui::MainWindow
+class MainWindow : public QMainWindow, public Ui::MainWindow, public InputEventHandler
 {
 	Q_OBJECT
 
 public:
-	static void requestOpenFile(const QString &filename);
-
 	QString fileName;
 
 	class Preferences *prefs;
 
 	QTimer *animate_timer;
-	double tval, fps, fsteps;
+	int anim_step;
+	int anim_numsteps;
+	double anim_tval;
+	bool anim_dumping;
+	int anim_dump_start_step;
 
 	QTimer *autoReloadTimer;
 	std::string autoReloadId;
 	QTimer *waitAfterReloadTimer;
-
 	QTime renderingTime;
 
-	ModuleContext top_ctx;
+	BuiltinContext top_ctx;
 	FileModule *root_module;      // Result of parsing
-	ModuleInstantiation root_inst;    // Top level instance
+	FileModule *parsed_module;		// Last parse for include list
+	ModuleInstantiation root_inst;	// Top level instance
 	AbstractNode *absolute_root_node; // Result of tree evaluation
-	AbstractNode *root_node;          // Root if the root modifier (!) is used
+	AbstractNode *root_node;		  // Root if the root modifier (!) is used
 	Tree tree;
 
-	shared_ptr<class CSGTerm> root_raw_term;           // Result of CSG term rendering
-	shared_ptr<CSGTerm> root_norm_term;          // Normalized CSG products
-	class CSGChain *root_chain;
 #ifdef ENABLE_CGAL
 	shared_ptr<const class Geometry> root_geom;
 	class CGALRenderer *cgalRenderer;
@@ -62,69 +58,72 @@ public:
 #endif
 	class ThrownTogetherRenderer *thrownTogetherRenderer;
 
-	std::vector<shared_ptr<CSGTerm> > highlight_terms;
-	CSGChain *highlights_chain;
-	std::vector<shared_ptr<CSGTerm> > background_terms;
-	CSGChain *background_chain;
 	QString last_compiled_doc;
 
 	QAction *actionRecentFile[UIUtils::maxRecentFiles];
-        QMap<QString, QString> knownFileExtensions;
+	QMap<QString, QString> knownFileExtensions;
+
+	QLabel *versionLabel;
+	QWidget *editorDockTitleWidget;
+	QWidget *consoleDockTitleWidget;
+	QWidget *parameterDockTitleWidget;
 
-        QLabel *versionLabel;
-        QWidget *editorDockTitleWidget;
-        QWidget *consoleDockTitleWidget;
-        
 	QString editortype;	
 	bool useScintilla;
 
-        int compileErrors;
-        int compileWarnings;
+	int compileErrors;
+	int compileWarnings;
 
 	MainWindow(const QString &filename);
 	~MainWindow();
 
 protected:
-	void closeEvent(QCloseEvent *event);
+	void closeEvent(QCloseEvent *event) override;
 
 private slots:
-	void updatedFps();
+	void updatedAnimTval();
+	void updatedAnimFps();
+	void updatedAnimSteps();
+	void updatedAnimDump(bool checked);
 	void updateTVal();
-        void updateMdiMode(bool mdi);
-        void updateUndockMode(bool undockMode);
-        void updateReorderMode(bool reorderMode);
+	void updateMdiMode(bool mdi);
+	void updateUndockMode(bool undockMode);
+	void updateReorderMode(bool reorderMode);
 	void setFileName(const QString &filename);
 	void setFont(const QString &family, uint size);
 	void setColorScheme(const QString &cs);
 	void showProgress();
 	void openCSGSettingsChanged();
 	void consoleOutput(const QString &msg);
+	void updateActionUndoState();
 
 private:
-        void initActionIcon(QAction *action, const char *darkResource, const char *lightResource);
-	void openFile(const QString &filename);
-        void handleFileDrop(const QString &filename);
+	void initActionIcon(QAction *action, const char *darkResource, const char *lightResource);
+	void handleFileDrop(const QString &filename);
 	void refreshDocument();
-        void updateCamera();
+	void updateCamera(const class FileContext &ctx);
 	void updateTemporalVariables();
 	bool fileChangedOnDisk();
-	void compileTopLevelDocument();
-        void updateCompileResult();
-	void compile(bool reload, bool forcedone = false);
-	void compileCSG(bool procevents);
+	void parseTopLevelDocument(bool rebuildParameterWidget);
+	void updateCompileResult();
+	void compile(bool reload, bool forcedone = false, bool rebuildParameterWidget=true);
+	void compileCSG();
 	bool maybeSave();
+	void saveError(const QIODevice &file, const std::string &msg);
 	bool checkEditorModified();
 	QString dumpCSGTree(AbstractNode *root);
 	static void consoleOutput(const std::string &msg, void *userdata);
+	static void noOutput(const std::string &, void*) {};  // /dev/null
 	void loadViewSettings();
 	void loadDesignSettings();
+	void updateWindowSettings(bool console, bool editor, bool customizer, bool toolbar);
 	void saveBackup();
 	void writeBackup(class QFile *file);
-	QString get2dExportFilename(QString format, QString extension);
 	void show_examples();
 	void setDockWidgetTitle(QDockWidget *dockWidget, QString prefix, bool topLevel);
-        void addKeyboardShortCut(const QList<QAction *> &actions);
-        void updateStatusBar(class ProgressWidget *progressWidget);
+	void addKeyboardShortCut(const QList<QAction *> &actions);
+	void updateStatusBar(class ProgressWidget *progressWidget);
+	void exceptionCleanup();
 
 	EditorInterface *editor;
 
@@ -143,26 +142,30 @@ private slots:
 	void actionSaveAs();
 	void actionReload();
 	void actionShowLibraryFolder();
-        void convertTabsToSpaces();
+	void convertTabsToSpaces();
 
 	void instantiateRoot();
 	void compileDone(bool didchange);
 	void compileEnded();
+	void changeParameterWidget();
 
 private slots:
-	void pasteViewportTranslation();
-	void pasteViewportRotation();
+	void copyViewportTranslation();
+	void copyViewportRotation();
+	void copyViewportDistance();
 	void preferences();
 	void hideToolbars();
 	void hideEditor();
 	void hideConsole();
 	void showConsole();
+	void hideParameters();
 
 private slots:
 	void selectFindType(int);
-	void find();
+	void hideFind();
+	void showFind();
 	void findString(QString);
-	void findAndReplace();
+	void showFindAndReplace();
 	void findNext();
 	void findPrev();
 	void useSelectionForFind();
@@ -172,13 +175,17 @@ private slots:
 	// Mac OSX FindBuffer support
 	void findBufferChanged();
 	void updateFindBuffer(QString);
+	bool event(QEvent* event) override;
 protected:
-	virtual bool eventFilter(QObject* obj, QEvent *event);
+	bool eventFilter(QObject* obj, QEvent *event) override;
 
 private slots:
-	void actionRenderPreview();
+	void actionRenderPreview(bool rebuildParameterWidget=true);
 	void csgRender();
 	void csgReloadRender();
+	void action3DPrint();
+	void sendToOctoPrint();
+	void sendToPrintService();
 #ifdef ENABLE_CGAL
 	void actionRender();
 	void actionRenderDone(shared_ptr<const class Geometry>);
@@ -188,29 +195,52 @@ private slots:
 	void actionDisplayAST();
 	void actionDisplayCSGTree();
 	void actionDisplayCSGProducts();
-	void actionExport(export_type_e, const char *, const char *);
+	bool canExport(unsigned int dim);
+	void actionExport(FileFormat format, const char *type_name, const char *suffix, unsigned int dim);
 	void actionExportSTL();
+	void actionExport3MF();
 	void actionExportOFF();
 	void actionExportAMF();
 	void actionExportDXF();
 	void actionExportSVG();
 	void actionExportCSG();
 	void actionExportImage();
+	void actionCopyViewport();
 	void actionFlushCaches();
 
 public:
-	static QSet<MainWindow*> *getWindows();
 	void viewModeActionsUncheck();
 	void setCurrentOutput();
 	void clearCurrentOutput();
+	void hideCurrentOutput();
+  bool isEmpty();
+
+	void onAxisChanged(InputEventAxisChanged *event) override;
+	void onButtonChanged(InputEventButtonChanged *event) override;
+
+	void onTranslateEvent(InputEventTranslate *event) override;
+	void onRotateEvent(InputEventRotate *event) override;
+	void onRotate2Event(InputEventRotate2 *event) override;
+	void onActionEvent(InputEventAction *event) override;
+	void onZoomEvent(InputEventZoom *event) override;
+
+	void changedTopLevelConsole(bool);
+
+	QList<double> getTranslation() const;
+	QList<double> getRotation() const;
 
 public slots:
+	void openFile(const QString &filename);
 	void actionReloadRenderPreview();
-        void on_editorDock_visibilityChanged(bool);
-        void on_consoleDock_visibilityChanged(bool);
-        void on_toolButtonCompileResultClose_clicked();
-        void editorTopLevelChanged(bool);
-        void consoleTopLevelChanged(bool);
+	void on_editorDock_visibilityChanged(bool);
+	void on_consoleDock_visibilityChanged(bool);
+	void on_parameterDock_visibilityChanged(bool);
+	void on_toolButtonCompileResultClose_clicked();
+	void editorTopLevelChanged(bool);
+	void consoleTopLevelChanged(bool);
+	void parameterTopLevelChanged(bool);
+	void processEvents();
+
 #ifdef ENABLE_OPENCSG
 	void viewModePreview();
 #endif
@@ -234,12 +264,13 @@ public slots:
 	void viewCenter();
 	void viewPerspective();
 	void viewOrthogonal();
+	void viewTogglePerspective();
 	void viewResetView();
 	void viewAll();
 	void animateUpdateDocChanged();
 	void animateUpdate();
-	void dragEnterEvent(QDragEnterEvent *event);
-	void dropEvent(QDropEvent *event);
+	void dragEnterEvent(QDragEnterEvent *event) override;
+	void dropEvent(QDropEvent *event) override;
 	void helpAbout();
 	void helpHomepage();
 	void helpManual();
@@ -251,16 +282,21 @@ public slots:
 	void waitAfterReload();
 	void autoReloadSet(bool);
 	void setContentsChanged();
-	void showFontCacheDialog();
-	void hideFontCacheDialog();
 
 private:
+	bool network_progress_func(const double permille);
 	static void report_func(const class AbstractNode*, void *vp, int mark);
 	static bool mdiMode;
 	static bool undockMode;
 	static bool reorderMode;
-	static QSet<MainWindow*> *windows;
-	static class QProgressDialog *fontCacheDialog;
+	static const int tabStopWidth;
+	static QElapsedTimer *progressThrottle;
+
+	shared_ptr<class CSGNode> csgRoot;		   // Result of the CSGTreeEvaluator
+	shared_ptr<CSGNode> normalizedRoot;		  // Normalized CSG tree
+ 	shared_ptr<class CSGProducts> root_products;
+	shared_ptr<CSGProducts> highlights_products;
+	shared_ptr<CSGProducts> background_products;
 
 	char const * afterCompileSlot;
 	bool procevents;
@@ -269,6 +305,12 @@ private:
 	class CGALWorker *cgalworker;
 	QMutex consolemutex;
 	bool contentschanged; // Set if the source code has changes since the last render (F6)
+	time_t includes_mtime;   // latest include mod time
+	time_t deps_mtime;	  // latest dependency mod time
+	std::unordered_map<std::string, QString> export_paths; // for each file type, where it was exported to last
+	void clearExportPaths(); // clear exports paths when main file is changed by open, new, etc.
+	QString exportPath(const char *suffix); // look up the last export path and generate one if not found
+	int last_parser_error_pos; // last highlighted error position
 
 signals:
 	void highlightError(int);
@@ -279,14 +321,18 @@ class GuiLocker
 {
 public:
 	GuiLocker() {
-		gui_locked++;
+		GuiLocker::lock();
 	}
 	~GuiLocker() {
-		gui_locked--;
+		GuiLocker::unlock();
 	}
 	static bool isLocked() { return gui_locked > 0; }
-	static void lock() { gui_locked++; }
-	static void unlock() { gui_locked--; }
+	static void lock() {
+		gui_locked++;
+	}
+	static void unlock() {
+		gui_locked--;
+	}
 
 private:
  	static unsigned int gui_locked;
