@@ -1,6 +1,6 @@
 ﻿/*
 
-Create an NULL OpenGL context that doesnt actually use any OpenGL code,
+Create an NULL OpenGL context that doesn't actually use any OpenGL code,
 and can be compiled on a system without OpenGL.
 
 */
@@ -31,13 +31,23 @@ void offscreen_context_init(OffscreenContext &ctx, int width, int height)
 
 string offscreen_context_getinfo(OffscreenContext *ctx)
 {
-  return string("NULLGL");
+  const char *arch = "unknown";
+  if (sizeof(int*) == 4) arch = "32-bit";
+  else if (sizeof(int*) == 8) arch = "64-bit";
+#ifdef OPENSCAD_OS
+  auto OSInfo = OPENSCAD_OS;
+#else
+  auto OSInfo = "unknown";
+#endif
+  return STR("OS info: " << OSInfo 
+    << "\nMachine: " << arch << "\n");
 }
 
 OffscreenContext *create_offscreen_context(int w, int h)
 {
   OffscreenContext *ctx = new OffscreenContext;
   offscreen_context_init( *ctx, w, h );
+	return ctx;
 }
 
 bool teardown_offscreen_context(OffscreenContext *ctx)
