@@ -2,32 +2,21 @@
 
 #ifdef ENABLE_CGAL
 
-#ifdef _MSC_VER
-// see http://en.wikipedia.org/wiki/Stdint.h
-// and http://www.mpfr.org/mpfr-2.4.2/#stdint
-#include <boost/cstdint.hpp>
-using boost::intmax_t;
-using boost::uintmax_t;
-#endif
-
 // NDEBUG must be disabled when including CGAL headers, otherwise CGAL assertions
 // will not be thrown, causing OpenSCAD's CGAL error checking to fail.
 // To be on the safe side, this has to be done when including any CGAL header file.
 // FIXME: It might be possible to rewrite the error checking to get rid of this
 // requirement. kintel 20111206.
-#ifdef NDEBUG
-#define PREV_NDEBUG NDEBUG
+#pragma push_macro("NDEBUG")
 #undef NDEBUG
-#endif
-
-#include "CGAL_workaround_Mark_bounded_volumes.h" // This file must be included prior to CGAL/Nef_polyhedron_3.h
+#include "ext/CGAL/CGAL_workaround_Mark_bounded_volumes.h" // This file must be included prior to CGAL/Nef_polyhedron_3.h
 #include <CGAL/Gmpq.h>
 #include <CGAL/Extended_cartesian.h>
 #include <CGAL/Nef_polyhedron_2.h>
 #include <CGAL/Cartesian.h>
 #include <CGAL/Polyhedron_3.h>
 #include <CGAL/Nef_polyhedron_3.h>
-#include "CGAL_Nef3_workaround.h"
+#include "ext/CGAL/CGAL_Nef3_workaround.h"
 #include <CGAL/IO/Polyhedron_iostream.h>
 #include <CGAL/Exact_predicates_exact_constructions_kernel.h>
 #include <CGAL/Polygon_2.h>
@@ -39,6 +28,7 @@ using boost::uintmax_t;
 
 #include <CGAL/assertions_behaviour.h>
 #include <CGAL/exceptions.h>
+#pragma pop_macro("NDEBUG")
 
 typedef CGAL::Gmpq NT2;
 typedef CGAL::Extended_cartesian<NT2> CGAL_Kernel2;
@@ -59,6 +49,7 @@ typedef CGAL_Nef_polyhedron3::Aff_transformation_3 CGAL_Aff_transformation;
 typedef CGAL::Polyhedron_3<CGAL_Kernel3> CGAL_Polyhedron;
 
 typedef CGAL::Point_3<CGAL_Kernel3> CGAL_Point_3;
+typedef CGAL::Triangle_3<CGAL_Kernel3> CGAL_Triangle_3;
 typedef CGAL::Iso_cuboid_3<CGAL_Kernel3> CGAL_Iso_cuboid_3;
 typedef std::vector<CGAL_Point_3> CGAL_Polygon_3;
 
@@ -66,11 +57,6 @@ typedef std::vector<CGAL_Point_3> CGAL_Polygon_3;
 // CGAL_Nef_polyhedron2::Explorer::Point which is different than
 // CGAL_Kernel2::Point. Hence the suffix 'e'
 typedef CGAL_Nef_polyhedron2::Explorer::Point CGAL_Point_2e;
-typedef CGAL::Iso_rectangle_2<CGAL::Simple_cartesian<NT2> > CGAL_Iso_rectangle_2e;
-
-
-#ifdef PREV_NDEBUG
-#define NDEBUG PREV_NDEBUG
-#endif
+typedef CGAL::Iso_rectangle_2<CGAL::Simple_cartesian<NT2>> CGAL_Iso_rectangle_2e;
 
 #endif /* ENABLE_CGAL */
