@@ -26,6 +26,7 @@
 
 #include "cgaladvnode.h"
 #include "module.h"
+#include "ModuleInstantiation.h"
 #include "evalcontext.h"
 #include "builtin.h"
 #include "polyset.h"
@@ -37,84 +38,60 @@ using namespace boost::assign; // bring 'operator+=()' into scope
 class CgaladvModule : public AbstractModule
 {
 public:
-	cgaladv_type_e type;
-	CgaladvModule(cgaladv_type_e type) : type(type) { }
-	virtual AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const;
+	CgaladvType type;
+	CgaladvModule(CgaladvType type) : type(type) { }
+	AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const override;
 };
 
 AbstractNode *CgaladvModule::instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const
 {
-	CgaladvNode *node = new CgaladvNode(inst, type);
+	auto node = new CgaladvNode(inst, type);
 
 	AssignmentList args;
 
-	if (type == MINKOWSKI)
+	if (type == CgaladvType::MINKOWSKI)
 		args += Assignment("convexity");
 
-	if (type == GLIDE)
-		args += Assignment("path"), Assignment("convexity");
-
-	if (type == SUBDIV)
-		args += Assignment("type"), Assignment("level"), Assignment("convexity");
-
-	if (type == RESIZE)
+	if (type == CgaladvType::RESIZE)
 		args += Assignment("newsize"), Assignment("auto");
 
 	Context c(ctx);
-	c.setVariables(args, evalctx);
+	c.setVariables(evalctx, args);
 	inst->scope.apply(*evalctx);
 
-	ValuePtr convexity = ValuePtr::undefined;
-	ValuePtr path = ValuePtr::undefined;
-	ValuePtr subdiv_type = ValuePtr::undefined;
-	ValuePtr level = ValuePtr::undefined;
+	auto convexity = ValuePtr::undefined;
+	auto path = ValuePtr::undefined;
 	
-	if (type == MINKOWSKI) {
+	if (type == CgaladvType::MINKOWSKI) {
 		convexity = c.lookup_variable("convexity", true);
 	}
 
-	if (type == GLIDE) {
-		convexity = c.lookup_variable("convexity", true);
-		path = c.lookup_variable("path", false);
-	}
-
-	if (type == SUBDIV) {
-		convexity = c.lookup_variable("convexity", true);
-		subdiv_type = c.lookup_variable("type", false);
-		level = c.lookup_variable("level", true);
-	}
-
-	if (type == RESIZE) {
-		ValuePtr ns = c.lookup_variable("newsize");
+	if (type == CgaladvType::RESIZE) {
+		auto ns = c.lookup_variable("newsize");
 		node->newsize << 0,0,0;
-		if ( ns->type() == Value::VECTOR ) {
+		if ( ns->type() == Value::ValueType::VECTOR ) {
 			const Value::VectorType &vs = ns->toVector();
 			if ( vs.size() >= 1 ) node->newsize[0] = vs[0]->toDouble();
 			if ( vs.size() >= 2 ) node->newsize[1] = vs[1]->toDouble();
 			if ( vs.size() >= 3 ) node->newsize[2] = vs[2]->toDouble();
 		}
-		ValuePtr autosize = c.lookup_variable("auto");
+		auto autosize = c.lookup_variable("auto");
 		node->autosize << false, false, false;
-		if ( autosize->type() == Value::VECTOR ) {
+		if ( autosize->type() == Value::ValueType::VECTOR ) {
 			const Value::VectorType &va = autosize->toVector();
 			if ( va.size() >= 1 ) node->autosize[0] = va[0]->toBool();
 			if ( va.size() >= 2 ) node->autosize[1] = va[1]->toBool();
 			if ( va.size() >= 3 ) node->autosize[2] = va[2]->toBool();
 		}
-		else if ( autosize->type() == Value::BOOL ) {
+		else if ( autosize->type() == Value::ValueType::BOOL ) {
 			node->autosize << autosize->toBool(),autosize->toBool(),autosize->toBool();
 		}
 	}
 
-	node->convexity = (int)convexity->toDouble();
+	node->convexity = static_cast<int>(convexity->toDouble());
 	node->path = path;
-	node->subdiv_type = subdiv_type->toString();
-	node->level = (int)level->toDouble();
 
-	if (node->level <= 1)
-		node->level = 1;
-
-	std::vector<AbstractNode *> instantiatednodes = inst->instantiateChildren(evalctx);
+	auto instantiatednodes = inst->instantiateChildren(evalctx);
 	node->children.insert(node->children.end(), instantiatednodes.begin(), instantiatednodes.end());
 
 	return node;
@@ -123,19 +100,13 @@ AbstractNode *CgaladvModule::instantiate(const Context *ctx, const ModuleInstant
 std::string CgaladvNode::name() const
 {
 	switch (this->type) {
-	case MINKOWSKI:
+	case CgaladvType::MINKOWSKI:
 		return "minkowski";
 		break;
-	case GLIDE:
-		return "glide";
-		break;
-	case SUBDIV:
-		return "subdiv";
-		break;
-	case HULL:
+	case CgaladvType::HULL:
 		return "hull";
 		break;
-	case RESIZE:
+	case CgaladvType::RESIZE:
 		return "resize";
 		break;
 	default:
@@ -146,23 +117,17 @@ std::string CgaladvNode::name() const
 
 std::string CgaladvNode::toString() const
 {
-	std::stringstream stream;
+	std::ostringstream stream;
 
 	stream << this->name();
 	switch (type) {
-	case MINKOWSKI:
+	case CgaladvType::MINKOWSKI:
 		stream << "(convexity = " << this->convexity << ")";
 		break;
-	case GLIDE:
-		stream << "(path = " << *this->path << ", convexity = " << this->convexity << ")";
-		break;
-	case SUBDIV:
-		stream << "(level = " << this->level << ", convexity = " << this->convexity << ")";
-		break;
-	case HULL:
+	case CgaladvType::HULL:
 		stream << "()";
 		break;
-	case RESIZE:
+	case CgaladvType::RESIZE:
 		stream << "(newsize = ["
 		  << this->newsize[0] << "," << this->newsize[1] << "," << this->newsize[2] << "]"
 		  << ", auto = ["
@@ -178,9 +143,7 @@ std::string CgaladvNode::toString() const
 
 void register_builtin_cgaladv()
 {
-	Builtins::init("minkowski", new CgaladvModule(MINKOWSKI));
-	Builtins::init("glide", new CgaladvModule(GLIDE));
-	Builtins::init("subdiv", new CgaladvModule(SUBDIV));
-	Builtins::init("hull", new CgaladvModule(HULL));
-	Builtins::init("resize", new CgaladvModule(RESIZE));
+	Builtins::init("minkowski", new CgaladvModule(CgaladvType::MINKOWSKI));
+	Builtins::init("hull", new CgaladvModule(CgaladvType::HULL));
+	Builtins::init("resize", new CgaladvModule(CgaladvType::RESIZE));
 }
