@@ -1,9 +1,11 @@
 #include "colormap.h"
-#include "boosty.h"
 #include "printutils.h"
 #include "PlatformUtils.h"
 
 #include <boost/property_tree/json_parser.hpp>
+#include <boost/filesystem.hpp>
+
+namespace fs=boost::filesystem;
 
 static const char *DEFAULT_COLOR_SCHEME_NAME = "Cornfield";
 
@@ -36,38 +38,40 @@ RenderColorScheme::RenderColorScheme() : _path("")
 	_index = 1000;
 	_show_in_gui = true;
 
-	_color_scheme.insert(ColorScheme::value_type(BACKGROUND_COLOR, Color4f(0xff, 0xff, 0xe5)));
-	_color_scheme.insert(ColorScheme::value_type(OPENCSG_FACE_FRONT_COLOR, Color4f(0xf9, 0xd7, 0x2c)));
-	_color_scheme.insert(ColorScheme::value_type(OPENCSG_FACE_BACK_COLOR, Color4f(0x9d, 0xcb, 0x51)));
-	_color_scheme.insert(ColorScheme::value_type(CGAL_FACE_FRONT_COLOR, Color4f(0xf9, 0xd7, 0x2c)));
-	_color_scheme.insert(ColorScheme::value_type(CGAL_FACE_2D_COLOR, Color4f(0x00, 0xbf, 0x99)));
-	_color_scheme.insert(ColorScheme::value_type(CGAL_FACE_BACK_COLOR, Color4f(0x9d, 0xcb, 0x51)));
-	_color_scheme.insert(ColorScheme::value_type(CGAL_EDGE_FRONT_COLOR, Color4f(0xff, 0xec, 0x5e)));
-	_color_scheme.insert(ColorScheme::value_type(CGAL_EDGE_BACK_COLOR, Color4f(0xab, 0xd8, 0x56)));
-	_color_scheme.insert(ColorScheme::value_type(CGAL_EDGE_2D_COLOR, Color4f(0xff, 0x00, 0x00)));
-	_color_scheme.insert(ColorScheme::value_type(CROSSHAIR_COLOR, Color4f(0x80, 0x00, 0x00)));
+	_color_scheme.insert(ColorScheme::value_type(RenderColor::BACKGROUND_COLOR, Color4f(0xff, 0xff, 0xe5)));
+	_color_scheme.insert(ColorScheme::value_type(RenderColor::AXES_COLOR, Color4f(0x00, 0x00, 0x00)));
+	_color_scheme.insert(ColorScheme::value_type(RenderColor::OPENCSG_FACE_FRONT_COLOR, Color4f(0xf9, 0xd7, 0x2c)));
+	_color_scheme.insert(ColorScheme::value_type(RenderColor::OPENCSG_FACE_BACK_COLOR, Color4f(0x9d, 0xcb, 0x51)));
+	_color_scheme.insert(ColorScheme::value_type(RenderColor::CGAL_FACE_FRONT_COLOR, Color4f(0xf9, 0xd7, 0x2c)));
+	_color_scheme.insert(ColorScheme::value_type(RenderColor::CGAL_FACE_2D_COLOR, Color4f(0x00, 0xbf, 0x99)));
+	_color_scheme.insert(ColorScheme::value_type(RenderColor::CGAL_FACE_BACK_COLOR, Color4f(0x9d, 0xcb, 0x51)));
+	_color_scheme.insert(ColorScheme::value_type(RenderColor::CGAL_EDGE_FRONT_COLOR, Color4f(0xff, 0xec, 0x5e)));
+	_color_scheme.insert(ColorScheme::value_type(RenderColor::CGAL_EDGE_BACK_COLOR, Color4f(0xab, 0xd8, 0x56)));
+	_color_scheme.insert(ColorScheme::value_type(RenderColor::CGAL_EDGE_2D_COLOR, Color4f(0xff, 0x00, 0x00)));
+	_color_scheme.insert(ColorScheme::value_type(RenderColor::CROSSHAIR_COLOR, Color4f(0x80, 0x00, 0x00)));
 }
 
 RenderColorScheme::RenderColorScheme(fs::path path) : _path(path)
 {
     try {
-	boost::property_tree::read_json(boosty::stringy(path).c_str(), pt);
+	boost::property_tree::read_json(path.generic_string().c_str(), pt);
 	_name = pt.get<std::string>("name");
 	_index = pt.get<int>("index");
 	_show_in_gui = pt.get<bool>("show-in-gui");
 	
-	addColor(BACKGROUND_COLOR, "background");
-	addColor(OPENCSG_FACE_FRONT_COLOR, "opencsg-face-front");
-	addColor(OPENCSG_FACE_BACK_COLOR, "opencsg-face-back");
-	addColor(CGAL_FACE_FRONT_COLOR, "cgal-face-front");
-	addColor(CGAL_FACE_2D_COLOR, "cgal-face-2d");
-	addColor(CGAL_FACE_BACK_COLOR, "cgal-face-back");
-	addColor(CGAL_EDGE_FRONT_COLOR, "cgal-edge-front");
-	addColor(CGAL_EDGE_BACK_COLOR, "cgal-edge-back");
-	addColor(CGAL_EDGE_2D_COLOR, "cgal-edge-2d");
-	addColor(CROSSHAIR_COLOR, "crosshair");
+	addColor(RenderColor::BACKGROUND_COLOR, "background");
+	addColor(RenderColor::AXES_COLOR, "axes-color");
+	addColor(RenderColor::OPENCSG_FACE_FRONT_COLOR, "opencsg-face-front");
+	addColor(RenderColor::OPENCSG_FACE_BACK_COLOR, "opencsg-face-back");
+	addColor(RenderColor::CGAL_FACE_FRONT_COLOR, "cgal-face-front");
+	addColor(RenderColor::CGAL_FACE_2D_COLOR, "cgal-face-2d");
+	addColor(RenderColor::CGAL_FACE_BACK_COLOR, "cgal-face-back");
+	addColor(RenderColor::CGAL_EDGE_FRONT_COLOR, "cgal-edge-front");
+	addColor(RenderColor::CGAL_EDGE_BACK_COLOR, "cgal-edge-back");
+	addColor(RenderColor::CGAL_EDGE_2D_COLOR, "cgal-edge-2d");
+	addColor(RenderColor::CROSSHAIR_COLOR, "crosshair");
     } catch (const std::exception & e) {
-			PRINTB("Error reading color scheme file '%s': %s", boosty::stringy(path).c_str() % e.what());
+			PRINTB("Error reading color scheme file '%s': %s", path.generic_string().c_str() % e.what());
 	_error = e.what();
 	_name = "";
 	_index = 0;
@@ -140,7 +144,7 @@ ColorMap *ColorMap::inst(bool erase)
 	static ColorMap *instance = new ColorMap;
 	if (erase) {
 		delete instance;
-		instance = NULL;
+		instance = nullptr;
 	}
 	return instance;
 }
@@ -175,7 +179,7 @@ const ColorScheme *ColorMap::findColorScheme(const std::string &name) const
 	}
     }
 
-    return NULL;
+    return nullptr;
 }
 
 void ColorMap::dump() const
@@ -238,7 +242,7 @@ Color4f ColorMap::getColorHSV(const Color4f &col)
 Color4f ColorMap::getContrastColor(const Color4f &col)
 {
 	Color4f hsv = ColorMap::getColorHSV(col);
-	float Y = 0.2126 * col[0] + 0.7152 * col[1] + 0.0722 * col[2];
+	float Y = 0.2126f * col[0] + 0.7152f * col[1] + 0.0722f * col[2];
 	float S = hsv[1];
 
 	if (S < 0.5) {
@@ -261,7 +265,7 @@ void ColorMap::enumerateColorSchemesInPath(colorscheme_set_t &result_set, const
 {
     const fs::path color_schemes = basePath / "color-schemes" / "render";
 
-    PRINTDB("Enumerating color schemes from '%s'", boosty::stringy(color_schemes).c_str());
+    PRINTDB("Enumerating color schemes from '%s'", color_schemes.generic_string().c_str());
     
     fs::directory_iterator end_iter;
     
@@ -277,8 +281,8 @@ void ColorMap::enumerateColorSchemesInPath(colorscheme_set_t &result_set, const
 	    }
 	    
 	    RenderColorScheme *colorScheme = new RenderColorScheme(path);
-	    if (colorScheme->valid() && (findColorScheme(colorScheme->name()) == 0)) {
-		result_set.insert(colorscheme_set_t::value_type(colorScheme->index(), boost::shared_ptr<RenderColorScheme>(colorScheme)));
+	    if (colorScheme->valid() && (findColorScheme(colorScheme->name()) == nullptr)) {
+		result_set.insert(colorscheme_set_t::value_type(colorScheme->index(), shared_ptr<RenderColorScheme>(colorScheme)));
 		PRINTDB("Found file '%s' with color scheme '%s' and index %d",
 			colorScheme->path() % colorScheme->name() % colorScheme->index());
 	    } else {
@@ -295,7 +299,7 @@ ColorMap::colorscheme_set_t ColorMap::enumerateColorSchemes()
 
     RenderColorScheme *defaultColorScheme = new RenderColorScheme();
     result_set.insert(colorscheme_set_t::value_type(defaultColorScheme->index(),
-	    boost::shared_ptr<RenderColorScheme>(defaultColorScheme)));
+	    shared_ptr<RenderColorScheme>(defaultColorScheme)));
     enumerateColorSchemesInPath(result_set, PlatformUtils::resourceBasePath());
     enumerateColorSchemesInPath(result_set, PlatformUtils::userConfigPath());
     
