@@ -24,39 +24,36 @@
  *
  */
 
-#include <boost/foreach.hpp>
 #include "module.h"
+#include "ModuleInstantiation.h"
 #include "node.h"
 #include "evalcontext.h"
 #include "modcontext.h"
 #include "expression.h"
 #include "builtin.h"
 #include "printutils.h"
-#include <sstream>
-#include "mathc99.h"
-
-
-#define foreach BOOST_FOREACH
-
+#include <cstdint>
 
 class ControlModule : public AbstractModule
 {
 public: // types
-	enum Type {
+	enum class Type {
 		CHILD,
 		CHILDREN,
 		ECHO,
+		ASSERT,
 		ASSIGN,
 		FOR,
+		LET,
 		INT_FOR,
 		IF
     };
 public: // methods
-	ControlModule(Type type)
-		: type(type)
-	{ }
+	ControlModule(Type type) : type(type) { }
 
-	virtual AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const;
+	ControlModule(Type type, const Feature& feature) : AbstractModule(feature), type(type) { }
+
+	AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const override;
 
 	static void for_eval(AbstractNode &node, const ModuleInstantiation &inst, size_t l, 
 						 const Context *ctx, const EvalContext *evalctx);
@@ -77,11 +74,11 @@ void ControlModule::for_eval(AbstractNode &node, const ModuleInstantiation &inst
 		const std::string &it_name = evalctx->getArgName(l);
 		ValuePtr it_values = evalctx->getArgValue(l, ctx);
 		Context c(ctx);
-		if (it_values->type() == Value::RANGE) {
+		if (it_values->type() == Value::ValueType::RANGE) {
 			RangeType range = it_values->toRange();
-			boost::uint32_t steps = range.numValues();
+			uint32_t steps = range.numValues();
 			if (steps >= 10000) {
-				PRINTB("WARNING: Bad range parameter in for statement: too many elements (%lu).", steps);
+				PRINTB("WARNING: Bad range parameter in for statement: too many elements (%lu), %s", steps % inst.location().toRelativeString(ctx->documentPath()));
 			} else {
 				for (RangeType::iterator it = range.begin();it != range.end();it++) {
 					c.set_variable(it_name, ValuePtr(*it));
@@ -89,13 +86,19 @@ void ControlModule::for_eval(AbstractNode &node, const ModuleInstantiation &inst
 				}
 			}
 		}
-		else if (it_values->type() == Value::VECTOR) {
+		else if (it_values->type() == Value::ValueType::VECTOR) {
 			for (size_t i = 0; i < it_values->toVector().size(); i++) {
 				c.set_variable(it_name, it_values->toVector()[i]);
 				for_eval(node, inst, l+1, &c, evalctx);
 			}
 		}
-		else if (it_values->type() != Value::UNDEFINED) {
+                else if (it_values->type() == Value::ValueType::STRING) {
+                        utf8_split(it_values->toString(), [&](ValuePtr v) {
+                            c.set_variable(it_name, v);
+                            for_eval(node, inst, l+1, &c, evalctx);
+                        });
+                }
+		else if (it_values->type() != Value::ValueType::UNDEFINED) {
 			c.set_variable(it_name, it_values);
 			for_eval(node, inst, l+1, &c, evalctx);
 		}
@@ -103,8 +106,8 @@ void ControlModule::for_eval(AbstractNode &node, const ModuleInstantiation &inst
 		// At this point, the for loop variables have been set and we can initialize
 		// the local scope (as they may depend on the for loop variables
 		Context c(ctx);
-		BOOST_FOREACH(const Assignment &ass, inst.scope.assignments) {
-			c.set_variable(ass.first, ass.second->evaluate(&c));
+		for(const auto &ass : inst.scope.assignments) {
+			c.set_variable(ass.name, ass.expr->evaluate(&c));
 		}
 		
 		std::vector<AbstractNode *> instantiatednodes = inst.instantiateChildren(&c);
@@ -125,50 +128,50 @@ const EvalContext* ControlModule::getLastModuleCtx(const EvalContext *evalctx)
 			if (modulectx->evalctx) {
 				return modulectx->evalctx;
 			}
-			return NULL;
+			return nullptr;
 		}
 		tmpc = tmpc->getParent();
 	}
-	return NULL;
+	return nullptr;
 }
 
 // static
 AbstractNode* ControlModule::getChild(const ValuePtr &value, const EvalContext* modulectx)
 {
-	if (value->type()!=Value::NUMBER) {
+	if (value->type()!=Value::ValueType::NUMBER) {
 		// Invalid parameter
 		// (e.g. first child of difference is invalid)
 		PRINTB("WARNING: Bad parameter type (%s) for children, only accept: empty, number, vector, range.", value->toString());
-		return NULL;
+		return nullptr;
 	}
 	double v;
 	if (!value->getDouble(v)) {
 		PRINTB("WARNING: Bad parameter type (%s) for children, only accept: empty, number, vector, range.", value->toString());
-		return NULL;
+		return nullptr;
 	}
 		
-	int n = trunc(v);
+	int n = static_cast<int>(trunc(v));
 	if (n < 0) {
 		PRINTB("WARNING: Negative children index (%d) not allowed", n);
-		return NULL; // Disallow negative child indices
+		return nullptr; // Disallow negative child indices
 	}
-	if (n>=(int)modulectx->numChildren()) {
+	if (n >= static_cast<int>(modulectx->numChildren())) {
 		// How to deal with negative objects in this case?
 		// (e.g. first child of difference is invalid)
 		PRINTB("WARNING: Children index (%d) out of bounds (%d children)"
 			, n % modulectx->numChildren());
-		return NULL;
+		return nullptr;
 	}
 	// OK
 	return modulectx->getChild(n)->evaluate(modulectx);
 }
 
-AbstractNode *ControlModule::instantiate(const Context* /*ctx*/, const ModuleInstantiation *inst, EvalContext *evalctx) const
+AbstractNode *ControlModule::instantiate(const Context* ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const
 {
-	AbstractNode *node = NULL;
+	AbstractNode *node = nullptr;
 
 	switch (this->type) {
-	case CHILD:	{
+	case Type::CHILD:	{
 		printDeprecation("child() will be removed in future releases. Use children() instead.");
 		int n = 0;
 		if (evalctx->numArgs() > 0) {
@@ -176,8 +179,8 @@ AbstractNode *ControlModule::instantiate(const Context* /*ctx*/, const ModuleIns
 			if (evalctx->getArgValue(0)->getDouble(v)) {
 				n = trunc(v);
 				if (n < 0) {
-					PRINTB("WARNING: Negative child index (%d) not allowed", n);
-					return NULL; // Disallow negative child indices
+					PRINTB("WARNING: Negative child index (%d) not allowed, %s", n % evalctx->loc.toRelativeString(ctx->documentPath()));
+					return nullptr; // Disallow negative child indices
 				}
 			}
 		}
@@ -185,8 +188,8 @@ AbstractNode *ControlModule::instantiate(const Context* /*ctx*/, const ModuleIns
 		// Find the last custom module invocation, which will contain
 		// an eval context with the children of the module invokation
 		const EvalContext *modulectx = getLastModuleCtx(evalctx);
-		if (modulectx==NULL) {
-			return NULL;
+		if (modulectx==nullptr) {
+			return nullptr;
 		}
 		// This will trigger if trying to invoke child from the root of any file
         if (n < (int)modulectx->numChildren()) {
@@ -195,26 +198,26 @@ AbstractNode *ControlModule::instantiate(const Context* /*ctx*/, const ModuleIns
 		else {
 			// How to deal with negative objects in this case?
             // (e.g. first child of difference is invalid)
-			PRINTB("WARNING: Child index (%d) out of bounds (%d children)", 
-				   n % modulectx->numChildren());
+			PRINTB("WARNING: Child index (%d) out of bounds (%d children), %s", 
+				   n % modulectx->numChildren() % evalctx->loc.toRelativeString(ctx->documentPath()));
 		}
 		return node;
 	}
 		break;
 
-	case CHILDREN: {
+	case Type::CHILDREN: {
 		const EvalContext *modulectx = getLastModuleCtx(evalctx);
-		if (modulectx==NULL) {
-			return NULL;
+		if (modulectx==nullptr) {
+			return nullptr;
 		}
 		// This will trigger if trying to invoke child from the root of any file
 		// assert(filectx->evalctx);
 		if (evalctx->numArgs()<=0) {
 			// no parameters => all children
-			AbstractNode* node = new AbstractNode(inst);
+			AbstractNode* node = new GroupNode(inst);
 			for (int n = 0; n < (int)modulectx->numChildren(); ++n) {
 				AbstractNode* childnode = modulectx->getChild(n)->evaluate(modulectx);
-				if (childnode==NULL) continue; // error
+				if (childnode==nullptr) continue; // error
 				node->children.push_back(childnode);
 			}
 			return node;
@@ -222,30 +225,30 @@ AbstractNode *ControlModule::instantiate(const Context* /*ctx*/, const ModuleIns
 		else if (evalctx->numArgs()>0) {
 			// one (or more ignored) parameter
 			ValuePtr value = evalctx->getArgValue(0);
-			if (value->type() == Value::NUMBER) {
+			if (value->type() == Value::ValueType::NUMBER) {
 				return getChild(value, modulectx);
 			}
-			else if (value->type() == Value::VECTOR) {
-				AbstractNode* node = new AbstractNode(inst);
+			else if (value->type() == Value::ValueType::VECTOR) {
+				AbstractNode* node = new GroupNode(inst);
 				const Value::VectorType& vect = value->toVector();
-				foreach (const ValuePtr &vectvalue, vect) {
+				for(const auto &vectvalue : vect) {
 					AbstractNode* childnode = getChild(vectvalue,modulectx);
-					if (childnode==NULL) continue; // error
+					if (childnode==nullptr) continue; // error
 					node->children.push_back(childnode);
 				}
 				return node;
 			}
-			else if (value->type() == Value::RANGE) {
+			else if (value->type() == Value::ValueType::RANGE) {
 				RangeType range = value->toRange();
-				boost::uint32_t steps = range.numValues();
+				uint32_t steps = range.numValues();
 				if (steps >= 10000) {
-					PRINTB("WARNING: Bad range parameter for children: too many elements (%lu).", steps);
-					return NULL;
+					PRINTB("WARNING: Bad range parameter for children: too many elements (%lu), %s", steps  % evalctx->loc.toRelativeString(ctx->documentPath()));
+					return nullptr;
 				}
-				AbstractNode* node = new AbstractNode(inst);
+				AbstractNode* node = new GroupNode(inst);
 				for (RangeType::iterator it = range.begin();it != range.end();it++) {
 					AbstractNode* childnode = getChild(ValuePtr(*it),modulectx); // with error cases
-					if (childnode==NULL) continue; // error
+					if (childnode==nullptr) continue; // error
 					node->children.push_back(childnode);
 				}
 				return node;
@@ -253,34 +256,44 @@ AbstractNode *ControlModule::instantiate(const Context* /*ctx*/, const ModuleIns
 			else {
 				// Invalid parameter
 				// (e.g. first child of difference is invalid)
-				PRINTB("WARNING: Bad parameter type (%s) for children, only accept: empty, number, vector, range.", value->toString());
-				return NULL;
+				PRINTB("WARNING: Bad parameter type (%s) for children, only accept: empty, number, vector, range, %s", value->toEchoString() % evalctx->loc.toRelativeString(ctx->documentPath()));
+				return nullptr;
 			}
 		}
-		return NULL;
+		return nullptr;
 	}
 		break;
 
-	case ECHO: {
-		node = new AbstractNode(inst);
-		std::stringstream msg;
-		msg << "ECHO: ";
-		for (size_t i = 0; i < inst->arguments.size(); i++) {
-			if (i > 0) msg << ", ";
-			if (!evalctx->getArgName(i).empty()) msg << evalctx->getArgName(i) << " = ";
-			ValuePtr val = evalctx->getArgValue(i);
-			if (val->type() == Value::STRING) {
-				msg << '"' << val->toString() << '"';
-			} else {
-				msg << val->toString();
-			}
-		}
-		PRINTB("%s", msg.str());
+	case Type::ECHO: {
+		node = new GroupNode(inst);
+		PRINTB("%s", STR("ECHO: " << *evalctx));
 	}
 		break;
 
-	case ASSIGN: {
-		node = new AbstractNode(inst);
+	case Type::ASSERT: {
+		node = new GroupNode(inst);
+
+		Context c(evalctx);
+		evaluate_assert(c, evalctx);
+		inst->scope.apply(c);
+		node->children = inst->instantiateChildren(&c);
+	}
+		break;
+
+	case Type::LET: {
+		node = new GroupNode(inst);
+		Context c(evalctx);
+
+		evalctx->assignTo(c);
+
+		inst->scope.apply(c);
+		std::vector<AbstractNode *> instantiatednodes = inst->instantiateChildren(&c);
+		node->children.insert(node->children.end(), instantiatednodes.begin(), instantiatednodes.end());
+	}
+		break;
+
+	case Type::ASSIGN: {
+		node = new GroupNode(inst);
 		// We create a new context to avoid parameters from influencing each other
 		// -> parallel evaluation. This is to be backwards compatible.
 		Context c(evalctx);
@@ -295,18 +308,18 @@ AbstractNode *ControlModule::instantiate(const Context* /*ctx*/, const ModuleIns
 	}
 		break;
 
-	case FOR:
-		node = new AbstractNode(inst);
+	case Type::FOR:
+		node = new GroupNode(inst);
 		for_eval(*node, *inst, 0, evalctx, evalctx);
 		break;
 
-	case INT_FOR:
+	case Type::INT_FOR:
 		node = new AbstractIntersectionNode(inst);
 		for_eval(*node, *inst, 0, evalctx, evalctx);
 		break;
 
-	case IF: {
-		node = new AbstractNode(inst);
+	case Type::IF: {
+		node = new GroupNode(inst);
 		const IfElseModuleInstantiation *ifelse = dynamic_cast<const IfElseModuleInstantiation*>(inst);
 		if (evalctx->numArgs() > 0 && evalctx->getArgValue(0)->toBool()) {
 			inst->scope.apply(*evalctx);
@@ -326,11 +339,13 @@ AbstractNode *ControlModule::instantiate(const Context* /*ctx*/, const ModuleIns
 
 void register_builtin_control()
 {
-	Builtins::init("child", new ControlModule(ControlModule::CHILD));
-	Builtins::init("children", new ControlModule(ControlModule::CHILDREN));
-	Builtins::init("echo", new ControlModule(ControlModule::ECHO));
-	Builtins::init("assign", new ControlModule(ControlModule::ASSIGN));
-	Builtins::init("for", new ControlModule(ControlModule::FOR));
-	Builtins::init("intersection_for", new ControlModule(ControlModule::INT_FOR));
-	Builtins::init("if", new ControlModule(ControlModule::IF));
+	Builtins::init("child", new ControlModule(ControlModule::Type::CHILD));
+	Builtins::init("children", new ControlModule(ControlModule::Type::CHILDREN));
+	Builtins::init("echo", new ControlModule(ControlModule::Type::ECHO));
+	Builtins::init("assert", new ControlModule(ControlModule::Type::ASSERT));
+	Builtins::init("assign", new ControlModule(ControlModule::Type::ASSIGN));
+	Builtins::init("for", new ControlModule(ControlModule::Type::FOR));
+	Builtins::init("let", new ControlModule(ControlModule::Type::LET));
+	Builtins::init("intersection_for", new ControlModule(ControlModule::Type::INT_FOR));
+	Builtins::init("if", new ControlModule(ControlModule::Type::IF));
 }
