@@ -24,11 +24,12 @@
  *
  */
 
-#include "csgnode.h"
+#include "csgops.h"
 
 #include "evalcontext.h"
 #include "module.h"
-#include "csgterm.h"
+#include "ModuleInstantiation.h"
+#include "csgnode.h"
 #include "builtin.h"
 #include <sstream>
 #include <assert.h>
@@ -38,33 +39,33 @@ class CsgModule : public AbstractModule
 public:
 	OpenSCADOperator type;
 	CsgModule(OpenSCADOperator type) : type(type) { }
-	virtual AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const;
+	AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const override;
 };
 
 AbstractNode *CsgModule::instantiate(const Context*, const ModuleInstantiation *inst, EvalContext *evalctx) const
 {
 	inst->scope.apply(*evalctx);
-	CsgNode *node = new CsgNode(inst, type);
-	std::vector<AbstractNode *> instantiatednodes = inst->instantiateChildren(evalctx);
+	auto node = new CsgOpNode(inst, type);
+	auto instantiatednodes = inst->instantiateChildren(evalctx);
 	node->children.insert(node->children.end(), instantiatednodes.begin(), instantiatednodes.end());
 	return node;
 }
 
-std::string CsgNode::toString() const
+std::string CsgOpNode::toString() const
 {
 	return this->name() + "()";
 }
 
-std::string CsgNode::name() const
+std::string CsgOpNode::name() const
 {
 	switch (this->type) {
-	case OPENSCAD_UNION:
+	case OpenSCADOperator::UNION:
 		return "union";
 		break;
-	case OPENSCAD_DIFFERENCE:
+	case OpenSCADOperator::DIFFERENCE:
 		return "difference";
 		break;
-	case OPENSCAD_INTERSECTION:
+	case OpenSCADOperator::INTERSECTION:
 		return "intersection";
 		break;
 	default:
@@ -75,8 +76,8 @@ std::string CsgNode::name() const
 
 void register_builtin_csgops()
 {
-	Builtins::init("union", new CsgModule(OPENSCAD_UNION));
-	Builtins::init("difference", new CsgModule(OPENSCAD_DIFFERENCE));
-	Builtins::init("intersection", new CsgModule(OPENSCAD_INTERSECTION));
+	Builtins::init("union", new CsgModule(OpenSCADOperator::UNION));
+	Builtins::init("difference", new CsgModule(OpenSCADOperator::DIFFERENCE));
+	Builtins::init("intersection", new CsgModule(OpenSCADOperator::INTERSECTION));
 }
 
