@@ -1,8 +1,7 @@
-#ifndef DXFDATA_H_
-#define DXFDATA_H_
+#pragma once
 
-#include "linalg.h"
 #include <vector>
+#include "linalg.h"
 
 class DxfData
 {
@@ -28,11 +27,7 @@ public:
 		}
 	};
 
-#ifdef __APPLE__
-	std::vector<Vector2d, Eigen::aligned_allocator<Vector2d> > points;
-#else
-	std::vector<Vector2d> points;
-#endif
+	VectorOfVector2d points;
 	std::vector<Path> paths;
 	std::vector<Dim> dims;
 
@@ -45,6 +40,5 @@ public:
 
 	void fixup_path_direction();
 	std::string dump() const;
+	class Polygon2d *toPolygon2d() const;
 };
-
-#endif
