@@ -1,5 +1,6 @@
 #include <stdio.h>
 #include <iostream>
+#include <sstream>
 #include <string>
 #include <map>
 
@@ -18,6 +19,7 @@ Feature::list_t Feature::feature_list;
  * argument to enable the option and for saving the option value in GUI
  * context.
  */
+const Feature Feature::ExperimentalInputDriverDBus("input-driver-dbus", "Enable DBus input drivers (requires restart)");
 
 Feature::Feature(const std::string &name, const std::string &description)
 	: enabled(false), name(name), description(description)
@@ -52,7 +54,7 @@ void Feature::enable(bool status)
     
 void Feature::enable_feature(const std::string &feature_name, bool status)
 {
-	map_t::iterator it = feature_map.find(feature_name);
+	auto it = feature_map.find(feature_name);
 	if (it != feature_map.end()) {
 		it->second->enable(status);
 	} else {
@@ -76,3 +78,21 @@ void Feature::dump_features()
 		std::cout << "Feature('" << it->first << "') = " << (it->second->is_enabled() ? "enabled" : "disabled") << std::endl;
 	}
 }
+
+ExperimentalFeatureException::ExperimentalFeatureException(const std::string &what_arg)
+    : EvaluationException(what_arg)
+{
+
+}
+
+ExperimentalFeatureException::~ExperimentalFeatureException() throw()
+{
+
+}
+
+void ExperimentalFeatureException::check(const Feature &feature)
+{
+	if (!feature.is_enabled()) {
+		throw ExperimentalFeatureException(STR("ERROR: Experimental feature not enabled: '" << feature.get_name() << "'. Please check preferences."));
+	}
+}
