@@ -30,7 +30,7 @@
 
  Speed Note:
 
- setFormat() is very slow. normally this doesnt matter because we
+ setFormat() is very slow. normally this doesn't matter because we
  only highlight a block or two at once. But when OpenSCAD first starts,
  QT automagically calls 'highlightBlock' on every single textblock in the file
  even if it's not visible in the window. On a large file (50,000 lines) this
@@ -208,7 +208,7 @@ void Highlighter::assignFormatsToTokens(const QString &s)
 		QStringList::iterator it;
 		for ( it = tokentypes[toktype].begin(); it < tokentypes[toktype].end(); ++it) {
 			QString token = *it;
-			//PRINTB("set format for %s: type %s", token.toStdString()%toktype.toStdString() );;
+			//PRINTB("set format for %s: type %s", token.toStdString()%toktype.toStdString() );
 			tokenFormats[ token ] = typeformats [ toktype ];
 		}
 	}
@@ -218,14 +218,16 @@ Highlighter::Highlighter(QTextDocument *parent)
 		: QSyntaxHighlighter(parent)
 {
 	tokentypes["operator"] << "=" << "!" << "&&" << "||" << "+" << "-" << "*" << "/" << "%" << "!" << "#" << ";";
-	tokentypes["math"] << "abs" << "sign" << "acos" << "asin" << "atan" << "atan2" << "sin" << "cos" << "floor" << "round" << "ceil" << "ln" << "log" << "lookup" << "min" << "max" << "pow" << "sqrt" << "exp" << "rands";
-	tokentypes["keyword"] << "module" << "function" << "for" << "intersection_for" << "if" << "assign" << "echo"<< "search" << "str" << "let";
+	tokentypes["math"] << "abs" << "sign" << "acos" << "asin" << "atan" << "atan2" << "sin" << "cos" << "floor" << "round" << "ceil"
+                           << "ln" << "log" << "lookup" << "min" << "max" << "pow" << "sqrt" << "exp" << "rands" << "chr" << "ord"
+                           << "is_undef" << "is_list" << "is_num" << "is_bool" << "is_string";
+	tokentypes["keyword"] << "module" << "function" << "for" << "intersection_for" << "if" << "assign" << "echo"<< "search" << "str" << "let" << "each" << "assert";
 	tokentypes["transform"] << "scale" << "translate" << "rotate" << "multmatrix" << "color" << "projection" << "hull" << "resize" << "mirror" << "minkowski";
 	tokentypes["csgop"]	<< "union" << "intersection" << "difference" << "render";
 	tokentypes["prim3d"] << "cube" << "cylinder" << "sphere" << "polyhedron";
 	tokentypes["prim2d"] << "square" << "polygon" << "circle";
 	tokentypes["import"] << "include" << "use" << "import_stl" << "import" << "import_dxf" << "dxf_dim" << "dxf_cross" << "surface";
-	tokentypes["special"] << "$children" << "child" << "children" << "$fn" << "$fa" << "$fs" << "$t" << "$vpt" << "$vpr" << "$vpd";
+	tokentypes["special"] << "$children" << "child" << "children" << "$fn" << "$fa" << "$fs" << "$t" << "$preview" << "$vpt" << "$vpr" << "$vpd";
 	tokentypes["extrude"] << "linear_extrude" << "rotate_extrude";
 	tokentypes["bracket"] << "[" << "]" << "(" << ")";
 	tokentypes["curlies"] << "{" << "}";
@@ -350,12 +352,12 @@ void Highlighter::highlightBlock(const QString &text)
 	}
 
 	// Quoting and Comments.
-	state_e state = (state_e) previousBlockState();
+	state_e state = static_cast<state_e>(previousBlockState());
 	int quote_esc_state = 0;
 	for (int n = 0; n < text.size(); ++n){
-		if (state == NORMAL){
+		if (state == state_e::NORMAL){
 			if (text[n] == '"'){
-				state = QUOTE;
+				state = state_e::QUOTE;
 				setFormat(n,1,quoteFormat);
 			} else if (text[n] == '/'){
 				if ( n+1 < text.size() && text[n+1] == '/'){
@@ -363,26 +365,26 @@ void Highlighter::highlightBlock(const QString &text)
 					break;
 				} else if ( n+1 < text.size() && text[n+1] == '*'){
 					setFormat(n++,2,commentFormat);
-					state = COMMENT;
+					state = state_e::COMMENT;
 				}
 			}
-		} else if (state == QUOTE){
+		} else if (state == state_e::QUOTE){
 			setFormat(n,1,quoteFormat);
 			if (quote_esc_state > 0)
 				quote_esc_state = 0;
 			else if (text[n] == '\\')
 				quote_esc_state = 1;
 			else if (text[n] == '"')
-				state = NORMAL;
-		} else if (state == COMMENT){
+				state = state_e::NORMAL;
+		} else if (state == state_e::COMMENT){
 			setFormat(n,1,commentFormat);
 			if (text[n] == '*' && n+1 < text.size() && text[n+1] == '/'){
 				setFormat(++n,1,commentFormat);
-				state = NORMAL;
+				state = state_e::NORMAL;
 			}
 		}
 	}
-	setCurrentBlockState((int) state);
+	setCurrentBlockState(static_cast<int>(state));
 
 	// Highlight an error. Do it last to 'overwrite' other formatting.
 	if (errorState) {
