@@ -13,21 +13,21 @@ class LaunchingScreen : public QDialog, public Ui::LaunchingScreen
 	
 public:
 	static LaunchingScreen *getDialog();
-	explicit LaunchingScreen(QWidget *parent = 0);
-	virtual ~LaunchingScreen();
-	QStringList selectedFiles();
+	explicit LaunchingScreen(QWidget *parent = nullptr);
+	~LaunchingScreen();
+	QStringList selectedFiles() const;
 
 public slots:
 	void openFile(const QString &filename);
 
 private slots:
-	void checkboxState(bool state);
+	void checkboxState(bool state) const;
 	void enableRecentButton(const QModelIndex &current, const QModelIndex &previous);
 	void enableExampleButton(QTreeWidgetItem *current, QTreeWidgetItem *previous);
 	void openUserFile();
 	void openRecent();
 	void openExample();
-	void openUserManualURL();
+	void openUserManualURL() const;
 	
 private:
 	void checkOpen(const QVariant &data);
