@@ -1,5 +1,5 @@
 #include "linalg.h"
-#include <boost/math/special_functions/fpclassify.hpp>
+#include <cmath>
 
 // FIXME: We can achieve better pruning by either:
 // o Recalculate the box based on the transformed object
@@ -31,7 +31,7 @@ bool matrix_contains_infinity( const Transform3d &m )
 {
   for (int i=0;i<m.matrix().rows();i++) {
     for (int j=0;j<m.matrix().cols();j++) {
-      if ((boost::math::isinf)(m(i,j))) return true;
+      if ((std::isinf)(m(i,j))) return true;
     }
   }
 	return false;
@@ -41,7 +41,7 @@ bool matrix_contains_nan( const Transform3d &m )
 {
   for (int i=0;i<m.matrix().rows();i++) {
     for (int j=0;j<m.matrix().cols();j++) {
-      if ((boost::math::isnan)(m(i,j))) return true;
+      if ((std::isnan)(m(i,j))) return true;
     }
   }
 	return false;
