@@ -27,14 +27,16 @@
 #include "linearextrudenode.h"
 
 #include "module.h"
+#include "ModuleInstantiation.h"
 #include "evalcontext.h"
 #include "printutils.h"
 #include "fileutils.h"
 #include "builtin.h"
 #include "calc.h"
 #include "polyset.h"
-#include "mathc99.h" 
+#include "handle_dep.h"
 
+#include <cmath>
 #include <sstream>
 #include <boost/assign/std/vector.hpp>
 using namespace boost::assign; // bring 'operator+=()' into scope
@@ -46,37 +48,39 @@ class LinearExtrudeModule : public AbstractModule
 {
 public:
 	LinearExtrudeModule() { }
-	virtual AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const;
+	AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const override;
 };
 
 AbstractNode *LinearExtrudeModule::instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const
 {
-	LinearExtrudeNode *node = new LinearExtrudeNode(inst);
+	auto node = new LinearExtrudeNode(inst);
 
-	AssignmentList args;
-	args += Assignment("file"), Assignment("layer"), Assignment("height"), Assignment("origin"), Assignment("scale"), Assignment("center"), Assignment("twist"), Assignment("slices");
+	AssignmentList args{Assignment("file"), Assignment("layer"), Assignment("height"), Assignment("origin"), Assignment("scale"), Assignment("center"), Assignment("twist"), Assignment("slices")};
+	AssignmentList optargs{Assignment("convexity")};
 
 	Context c(ctx);
-	c.setVariables(args, evalctx);
+	c.setVariables(evalctx, args, optargs);
 	inst->scope.apply(*evalctx);
 
 	node->fn = c.lookup_variable("$fn")->toDouble();
 	node->fs = c.lookup_variable("$fs")->toDouble();
 	node->fa = c.lookup_variable("$fa")->toDouble();
 
-	ValuePtr file = c.lookup_variable("file");
-	ValuePtr layer = c.lookup_variable("layer", true);
-	ValuePtr height = c.lookup_variable("height", true);
-	ValuePtr convexity = c.lookup_variable("convexity", true);
-	ValuePtr origin = c.lookup_variable("origin", true);
-	ValuePtr scale = c.lookup_variable("scale", true);
-	ValuePtr center = c.lookup_variable("center", true);
-	ValuePtr twist = c.lookup_variable("twist", true);
-	ValuePtr slices = c.lookup_variable("slices", true);
+	auto file = c.lookup_variable("file");
+	auto layer = c.lookup_variable("layer", true);
+	auto height = c.lookup_variable("height", true);
+	auto convexity = c.lookup_variable("convexity", true);
+	auto origin = c.lookup_variable("origin", true);
+	auto scale = c.lookup_variable("scale", true);
+	auto center = c.lookup_variable("center", true);
+	auto twist = c.lookup_variable("twist", true);
+	auto slices = c.lookup_variable("slices", true);
 
-	if (!file->isUndefined() && file->type() == Value::STRING) {
+	if (!file->isUndefined() && file->type() == Value::ValueType::STRING) {
 		printDeprecation("Support for reading files in linear_extrude will be removed in future releases. Use a child import() instead.");
-		node->filename = lookup_file(file->toString(), inst->path(), c.documentPath());
+		auto filename = lookup_file(file->toString(), inst->path(), c.documentPath());
+		node->filename = filename;
+		handle_dep(filename);
 	}
 
 	// if height not given, and first argument is a number,
@@ -84,21 +88,28 @@ AbstractNode *LinearExtrudeModule::instantiate(const Context *ctx, const ModuleI
 	if (c.lookup_variable("height")->isUndefined() &&
 			evalctx->numArgs() > 0 &&
 			evalctx->getArgName(0) == "") {
-		ValuePtr val = evalctx->getArgValue(0);
-		if (val->type() == Value::NUMBER) height = val;
+		auto val = evalctx->getArgValue(0);
+		if (val->type() == Value::ValueType::NUMBER) height = val;
 	}
 
 	node->layername = layer->isUndefined() ? "" : layer->toString();
 	node->height = 100;
-	height->getDouble(node->height);
-	node->convexity = (int)convexity->toDouble();
-	origin->getVec2(node->origin_x, node->origin_y);
+	height->getFiniteDouble(node->height);
+	node->convexity = static_cast<int>(convexity->toDouble());
+	bool originOk = origin->getVec2(node->origin_x, node->origin_y);
+	originOk &= std::isfinite(node->origin_x) && std::isfinite(node->origin_y);
+	if(origin!=ValuePtr::undefined && !originOk){
+		PRINTB("WARNING: linear_extrude(..., origin=%s) could not be converted, %s", origin->toEchoString() % evalctx->loc.toRelativeString(ctx->documentPath()));
+	}
 	node->scale_x = node->scale_y = 1;
-	scale->getDouble(node->scale_x);
-	scale->getDouble(node->scale_y);
-	scale->getVec2(node->scale_x, node->scale_y);
+	bool scaleOK = scale->getFiniteDouble(node->scale_x);
+	scaleOK &= scale->getFiniteDouble(node->scale_y);
+	scaleOK |= scale->getVec2(node->scale_x, node->scale_y, true);
+	if((origin!=ValuePtr::undefined) && (!scaleOK || !std::isfinite(node->scale_x) || !std::isfinite(node->scale_y))){
+		PRINTB("WARNING: linear_extrude(..., scale=%s) could not be converted, %s", scale->toEchoString() % evalctx->loc.toRelativeString(ctx->documentPath()));
+	}
 
-	if (center->type() == Value::BOOL)
+	if (center->type() == Value::ValueType::BOOL)
 		node->center = center->toBool();
 
 	if (node->height <= 0) node->height = 0;
@@ -109,22 +120,22 @@ AbstractNode *LinearExtrudeModule::instantiate(const Context *ctx, const ModuleI
 	if (node->scale_x < 0) node->scale_x = 0;
 	if (node->scale_y < 0) node->scale_y = 0;
 
-	if (slices->type() == Value::NUMBER) node->slices = (int)slices->toDouble();
+	double slicesVal = 0;
+	slices->getFiniteDouble(slicesVal);
+	node->slices = static_cast<int>(slicesVal);
 
-	if (twist->type() == Value::NUMBER) {
-		node->twist = twist->toDouble();
-		if (node->twist != 0.0) {
-			if (node->slices == 0) {
-				node->slices = (int)fmax(2, fabs(Calc::get_fragments_from_r(node->height,
-																																		node->fn, node->fs, node->fa) * node->twist / 360));
-			}
-			node->has_twist = true;
+	node->twist = 0.0;
+	twist->getFiniteDouble(node->twist);
+	if (node->twist != 0.0) {
+		if (node->slices == 0) {
+			node->slices = static_cast<int>(fmax(2, fabs(Calc::get_fragments_from_r(node->height, node->fn, node->fs, node->fa) * node->twist / 360)));
 		}
+		node->has_twist = true;
 	}
 	node->slices = std::max(node->slices, 1);
 
 	if (node->filename.empty()) {
-		std::vector<AbstractNode *> instantiatednodes = inst->instantiateChildren(evalctx);
+		auto instantiatednodes = inst->instantiateChildren(evalctx);
 		node->children.insert(node->children.end(), instantiatednodes.begin(), instantiatednodes.end());
 	}
 
@@ -133,7 +144,7 @@ AbstractNode *LinearExtrudeModule::instantiate(const Context *ctx, const ModuleI
 
 std::string LinearExtrudeNode::toString() const
 {
-	std::stringstream stream;
+	std::ostringstream stream;
 
 	stream << this->name() << "(";
 	if (!this->filename.empty()) { // Ignore deprecated parameters if empty 
@@ -142,10 +153,7 @@ std::string LinearExtrudeNode::toString() const
 			"file = " << this->filename << ", "
 			"layer = " << QuotedString(this->layername) << ", "
 			"origin = [" << this->origin_x << ", " << this->origin_y << "], "
-#ifndef OPENSCAD_TESTING
-			// timestamp is needed for caching, but disturbs the test framework
 			<< "timestamp = " << (fs::exists(path) ? fs::last_write_time(path) : 0) << ", "
-#endif
 			;
 	}
 	stream <<
