@@ -1,12 +1,12 @@
 #pragma once
 
 #include "node.h"
-#include "visitor.h"
 #include "value.h"
 
 class LinearExtrudeNode : public AbstractPolyNode
 {
 public:
+	VISITABLE();
 	LinearExtrudeNode(const ModuleInstantiation *mi) : AbstractPolyNode(mi) {
 		convexity = slices = 0;
 		fn = fs = fa = height = twist = 0;
@@ -14,11 +14,8 @@ public:
 		scale_x = scale_y = 1;
 		center = has_twist = false;
 	}
-  virtual Response accept(class State &state, Visitor &visitor) const {
-		return visitor.visit(state, *this);
-	}
-	virtual std::string toString() const;
-	virtual std::string name() const { return "linear_extrude"; }
+	std::string toString() const override;
+	std::string name() const override { return "linear_extrude"; }
 
 	int convexity, slices;
 	double fn, fs, fa, height, twist;
