@@ -1,15 +1,18 @@
 #include "modcontext.h"
-#include "module.h"
+#include "UserModule.h"
+#include "ModuleInstantiation.h"
 #include "expression.h"
 #include "function.h"
 #include "printutils.h"
 #include "builtin.h"
 #include "ModuleCache.h"
-
-#include <boost/foreach.hpp>
+#include <cmath>
+#ifdef DEBUG
+#include <boost/format.hpp>
+#endif
 
 ModuleContext::ModuleContext(const Context *parent, const EvalContext *evalctx)
-	: Context(parent), functions_p(NULL), modules_p(NULL), evalctx(evalctx)
+	: Context(parent), functions_p(nullptr), modules_p(nullptr), evalctx(evalctx)
 {
 }
 
@@ -23,7 +26,7 @@ void ModuleContext::evaluateAssignments(const AssignmentList &assignments)
 {
 	// First, assign all simple variables
 	std::list<std::string> undefined_vars;
- 	BOOST_FOREACH(const Assignment &ass, assignments) {
+ 	for (const auto &ass : assignments) {
 		ValuePtr tmpval = ass.second->evaluate(this);
 		if (tmpval->isUndefined()) undefined_vars.push_back(ass.first);
  		else this->set_variable(ass.first, tmpval);
@@ -32,8 +35,8 @@ void ModuleContext::evaluateAssignments(const AssignmentList &assignments)
 	// Variables which couldn't be evaluated in the first pass is attempted again,
   // to allow for initialization out of order
 
-	boost::unordered_map<std::string, Expression *> tmpass;
-	BOOST_FOREACH (const Assignment &ass, assignments) {
+	std::unordered_map<std::string, Expression *> tmpass;
+	for(const auto &ass : assignments) {
 		tmpass[ass.first] = ass.second;
 	}
 		
@@ -43,7 +46,7 @@ void ModuleContext::evaluateAssignments(const AssignmentList &assignments)
 		std::list<std::string>::iterator iter = undefined_vars.begin();
 		while (iter != undefined_vars.end()) {
 			std::list<std::string>::iterator curr = iter++;
-			boost::unordered_map<std::string, Expression *>::iterator found = tmpass.find(*curr);
+			std::unordered_map<std::string, Expression *>::iterator found = tmpass.find(*curr);
 			if (found != tmpass.end()) {
 				const Expression *expr = found->second;
 				ValuePtr tmpval = expr->evaluate(this);
@@ -57,76 +60,63 @@ void ModuleContext::evaluateAssignments(const AssignmentList &assignments)
 					undefined_vars.erase(curr);
 				}
 			}
-		}
+		}+-
 	}
 }
 #endif
 
-void ModuleContext::initializeModule(const class Module &module)
+void ModuleContext::initializeModule(const UserModule &module)
 {
-	this->setVariables(module.definition_arguments, evalctx);
+	this->setVariables(evalctx, module.definition_arguments, {}, true);
 	// FIXME: Don't access module members directly
 	this->functions_p = &module.scope.functions;
 	this->modules_p = &module.scope.modules;
-	BOOST_FOREACH(const Assignment &ass, module.scope.assignments) {
-		this->set_variable(ass.first, ass.second->evaluate(this));
+	for (const auto &ass : module.scope.assignments) {
+		if (ass.expr->isLiteral() && this->variables.find(ass.name) != this->variables.end()) {
+			std::string loc = ass.location().toRelativeString(this->documentPath());
+			PRINTB("WARNING: Module %s: Parameter %s is overwritten with a literal, %s", module.name % ass.name % loc);
+		}
+		this->set_variable(ass.name, ass.expr->evaluate(this));
 	}
 
 // Experimental code. See issue #399
 //	evaluateAssignments(module.scope.assignments);
 }
 
-/*!
-	Only used to initialize builtins for the top-level root context
-*/
-void ModuleContext::registerBuiltin()
+const UserFunction *ModuleContext::findLocalFunction(const std::string &name) const
 {
-	const LocalScope &scope = Builtins::instance()->getGlobalScope();
-
-	// FIXME: Don't access module members directly
-	this->functions_p = &scope.functions;
-	this->modules_p = &scope.modules;
-	BOOST_FOREACH(const Assignment &ass, scope.assignments) {
-		this->set_variable(ass.first, ass.second->evaluate(this));
-	}
-
-	this->set_constant("PI", ValuePtr(M_PI));
-}
-
-const AbstractFunction *ModuleContext::findLocalFunction(const std::string &name) const
-{
-	if (this->functions_p && this->functions_p->find(name) != this->functions_p->end()) {
-		AbstractFunction *f = this->functions_p->find(name)->second;
+ 	if (this->functions_p && this->functions_p->find(name) != this->functions_p->end()) {
+		auto f = this->functions_p->find(name)->second;
 		if (!f->is_enabled()) {
 			PRINTB("WARNING: Experimental builtin function '%s' is not enabled.", name);
-			return NULL;
+			return nullptr;
 		}
 		return f;
 	}
-	return NULL;
+	return nullptr;
 }
 
-const AbstractModule *ModuleContext::findLocalModule(const std::string &name) const
+const UserModule *ModuleContext::findLocalModule(const std::string &name) const
 {
 	if (this->modules_p && this->modules_p->find(name) != this->modules_p->end()) {
-		AbstractModule *m = this->modules_p->find(name)->second;
+		auto m = this->modules_p->find(name)->second;
 		if (!m->is_enabled()) {
 			PRINTB("WARNING: Experimental builtin module '%s' is not enabled.", name);
-			return NULL;
+			return nullptr;
 		}
-		std::string replacement = Builtins::instance()->isDeprecated(name);
+		auto replacement = Builtins::instance()->isDeprecated(name);
 		if (!replacement.empty()) {
 			PRINT_DEPRECATION("The %s() module will be removed in future releases. Use %s instead.", name % replacement);
 		}
 		return m;
 	}
-	return NULL;
+	return nullptr;
 }
 
 ValuePtr ModuleContext::evaluate_function(const std::string &name, 
 																												 const EvalContext *evalctx) const
 {
-	const AbstractFunction *foundf = findLocalFunction(name);
+	const auto foundf = findLocalFunction(name);
 	if (foundf) return foundf->evaluate(this, evalctx);
 
 	return Context::evaluate_function(name, evalctx);
@@ -134,7 +124,7 @@ ValuePtr ModuleContext::evaluate_function(const std::string &name,
 
 AbstractNode *ModuleContext::instantiate_module(const ModuleInstantiation &inst, EvalContext *evalctx) const
 {
-	const AbstractModule *foundm = this->findLocalModule(inst.name());
+	const auto foundm = this->findLocalModule(inst.name());
 	if (foundm) return foundm->instantiate(this, &inst, evalctx);
 
 	return Context::instantiate_module(inst, evalctx);
@@ -143,53 +133,52 @@ AbstractNode *ModuleContext::instantiate_module(const ModuleInstantiation &inst,
 #ifdef DEBUG
 std::string ModuleContext::dump(const AbstractModule *mod, const ModuleInstantiation *inst)
 {
-	std::stringstream s;
-	if (inst)
+	std::ostringstream s;
+	if (inst) {
 		s << boost::format("ModuleContext %p (%p) for %s inst (%p) ") % this % this->parent % inst->name() % inst;
-	else
+	}
+	else {
 		s << boost::format("ModuleContext: %p (%p)") % this % this->parent;
+	}
 	s << boost::format("  document path: %s") % this->document_path;
 	if (mod) {
-		const Module *m = dynamic_cast<const Module*>(mod);
+		const UserModule *m = dynamic_cast<const UserModule*>(mod);
 		if (m) {
 			s << "  module args:";
-			BOOST_FOREACH(const Assignment &arg, m->definition_arguments) {
-				s << boost::format("    %s = %s") % arg.first % variables[arg.first];
+			for(const auto &arg : m->definition_arguments) {
+				s << boost::format("    %s = %s") % arg.name % variables[arg.name];
 			}
 		}
 	}
 	typedef std::pair<std::string, ValuePtr> ValueMapType;
 	s << "  vars:";
-	BOOST_FOREACH(const ValueMapType &v, constants) {
+	for(const auto &v : constants) {
 		s << boost::format("    %s = %s") % v.first % v.second;
 	}
-	BOOST_FOREACH(const ValueMapType &v, variables) {
+	for(const auto &v : variables) {
 		s << boost::format("    %s = %s") % v.first % v.second;
 	}
-	BOOST_FOREACH(const ValueMapType &v, config_variables) {
+	for(const auto &v : config_variables) {
 		s << boost::format("    %s = %s") % v.first % v.second;
 	}
 	return s.str();
 }
 #endif
 
-FileContext::FileContext(const class FileModule &module, const Context *parent)
-	: ModuleContext(parent), usedlibs(module.usedlibs)
+FileContext::FileContext(const Context *parent) : ModuleContext(parent), usedlibs_p(nullptr)
 {
-	if (!module.modulePath().empty()) this->document_path = module.modulePath();
 }
 
 ValuePtr FileContext::sub_evaluate_function(const std::string &name, 
-																													 const EvalContext *evalctx,
-																													 FileModule *usedmod) const
-
+																						const EvalContext *evalctx,
+																						FileModule *usedmod) const
 {
-	FileContext ctx(*usedmod, this->parent);
+	FileContext ctx(this->parent);
 	ctx.initializeModule(*usedmod);
 	// FIXME: Set document path
 #ifdef DEBUG
 	PRINTDB("New lib Context for %s func:", name);
-	PRINTDB("%s",ctx.dump(NULL, NULL));
+	PRINTDB("%s",ctx.dump(nullptr, nullptr));
 #endif
 	return usedmod->scope.functions[name]->evaluate(&ctx, evalctx);
 }
@@ -197,12 +186,12 @@ ValuePtr FileContext::sub_evaluate_function(const std::string &name,
 ValuePtr FileContext::evaluate_function(const std::string &name, 
 																											 const EvalContext *evalctx) const
 {
-	const AbstractFunction *foundf = findLocalFunction(name);
+	const auto foundf = findLocalFunction(name);
 	if (foundf) return foundf->evaluate(this, evalctx);
 
-	BOOST_FOREACH(const FileModule::ModuleContainer::value_type &m, this->usedlibs) {
-		// usedmod is NULL if the library wasn't be compiled (error or file-not-found)
-		FileModule *usedmod = ModuleCache::instance()->lookup(m);
+	for (const auto &m : *this->usedlibs_p) {
+		// usedmod is nullptr if the library wasn't be compiled (error or file-not-found)
+		auto usedmod = ModuleCache::instance()->lookup(m);
 		if (usedmod && usedmod->scope.functions.find(name) != usedmod->scope.functions.end())
 			return sub_evaluate_function(name, evalctx, usedmod);
 	}
@@ -212,20 +201,20 @@ ValuePtr FileContext::evaluate_function(const std::string &name,
 
 AbstractNode *FileContext::instantiate_module(const ModuleInstantiation &inst, EvalContext *evalctx) const
 {
-	const AbstractModule *foundm = this->findLocalModule(inst.name());
+	const auto foundm = this->findLocalModule(inst.name());
 	if (foundm) return foundm->instantiate(this, &inst, evalctx);
 
-	BOOST_FOREACH(const FileModule::ModuleContainer::value_type &m, this->usedlibs) {
-		FileModule *usedmod = ModuleCache::instance()->lookup(m);
-		// usedmod is NULL if the library wasn't be compiled (error or file-not-found)
+	for (const auto &m : *this->usedlibs_p) {
+		auto usedmod = ModuleCache::instance()->lookup(m);
+		// usedmod is nullptr if the library wasn't be compiled (error or file-not-found)
 		if (usedmod &&
 				usedmod->scope.modules.find(inst.name()) != usedmod->scope.modules.end()) {
-			FileContext ctx(*usedmod, this->parent);
+			FileContext ctx(this->parent);
 			ctx.initializeModule(*usedmod);
 			// FIXME: Set document path
 #ifdef DEBUG
 			PRINTD("New file Context:");
-			PRINTDB("%s",ctx.dump(NULL, &inst));
+			PRINTDB("%s",ctx.dump(nullptr, &inst));
 #endif
 			return usedmod->scope.modules[inst.name()]->instantiate(&ctx, &inst, evalctx);
 		}
@@ -233,3 +222,15 @@ AbstractNode *FileContext::instantiate_module(const ModuleInstantiation &inst, E
 
 	return ModuleContext::instantiate_module(inst, evalctx);
 }
+
+void FileContext::initializeModule(const class FileModule &module)
+{
+	if (!module.modulePath().empty()) this->document_path = module.modulePath();
+	// FIXME: Don't access module members directly
+	this->usedlibs_p = &module.usedlibs;
+	this->functions_p = &module.scope.functions;
+	this->modules_p = &module.scope.modules;
+	for (const auto &ass : module.scope.assignments) {
+		this->set_variable(ass.name, ass.expr->evaluate(this));
+	}
+}
