@@ -26,7 +26,9 @@
 
 #pragma once
 
-extern class FileModule *parse(const char *text, const char *path, int debug);
+#include <boost/filesystem.hpp>
+
+extern bool parse(class FileModule *&module, const std::string& text, const std::string &filename, const std::string &mainFile, int debug);
 
 #include <string>
 extern std::string commandline_commands;
@@ -35,12 +37,5 @@ extern std::string commandline_commands;
 // doing this, use currentdir to get the original CWD.
 extern std::string currentdir;
 
-// Version number without any patch level indicator
-extern std::string openscad_shortversionnumber;
-// The full version number, e.g. 2014.03, 2015.03-1, 2014.12.23
-extern std::string openscad_versionnumber;
-// Version used for display, typically without patchlevel indicator,
-// but may include git commit id for snapshot builds
-extern std::string openscad_displayversionnumber;
-// Version used for detailed display
-extern std::string openscad_detailedversionnumber;
+// Custom argument parser
+std::pair<std::string, std::string> customSyntax(const std::string& s);
