@@ -15,4 +15,4 @@ void parser_init();
 fs::path search_libs(const fs::path &localpath);
 fs::path find_valid_path(const fs::path &sourcepath, 
                          const fs::path &localpath, 
-                         const std::vector<std::string> *openfilenames = NULL);
+                         const std::vector<std::string> *openfilenames = nullptr);
