@@ -26,14 +26,13 @@
 
 #include "projectionnode.h"
 #include "module.h"
+#include "ModuleInstantiation.h"
 #include "evalcontext.h"
 #include "printutils.h"
 #include "builtin.h"
-#include "visitor.h"
 #include "polyset.h"
 
 #include <assert.h>
-#include <sstream>
 #include <boost/assign/std/vector.hpp>
 using namespace boost::assign; // bring 'operator+=()' into scope
 
@@ -41,29 +40,30 @@ class ProjectionModule : public AbstractModule
 {
 public:
 	ProjectionModule() { }
-	virtual AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const;
+	AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const override;
 };
 
 AbstractNode *ProjectionModule::instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const
 {
-	ProjectionNode *node = new ProjectionNode(inst);
-
-	AssignmentList args;
-	args += Assignment("cut");
+	auto node = new ProjectionNode(inst);
 
+	AssignmentList args{Assignment("cut")};
+	AssignmentList optargs{Assignment("convexity")};
+	
 	Context c(ctx);
-	c.setVariables(args, evalctx);
+	c.setVariables(evalctx, args, optargs);
 	inst->scope.apply(*evalctx);
 
-	ValuePtr convexity = c.lookup_variable("convexity", true);
-	ValuePtr cut = c.lookup_variable("cut", true);
+	auto convexity = c.lookup_variable("convexity", true);
+	auto cut = c.lookup_variable("cut", true);
 
-	node->convexity = (int)convexity->toDouble();
+	node->convexity = static_cast<int>(convexity->toDouble());
 
-	if (cut->type() == Value::BOOL)
+	if (cut->type() == Value::ValueType::BOOL) {
 		node->cut_mode = cut->toBool();
+	}
 
-	std::vector<AbstractNode *> instantiatednodes = inst->instantiateChildren(evalctx);
+	auto instantiatednodes = inst->instantiateChildren(evalctx);
 	node->children.insert(node->children.end(), instantiatednodes.begin(), instantiatednodes.end());
 
 	return node;
@@ -71,12 +71,8 @@ AbstractNode *ProjectionModule::instantiate(const Context *ctx, const ModuleInst
 
 std::string ProjectionNode::toString() const
 {
-	std::stringstream stream;
-
-	stream << "projection(cut = " << (this->cut_mode ? "true" : "false")
-				 << ", convexity = " << this->convexity << ")";
-
-	return stream.str();
+	return STR("projection(cut = " << (this->cut_mode ? "true" : "false")
+						 << ", convexity = " << this->convexity << ")");
 }
 
 void register_builtin_projection()
