@@ -2,12 +2,14 @@
 #include <algorithm>
 #include <QString>
 #include <QChar>
-#include "boosty.h"
 #include "scintillaeditor.h"
 #include <Qsci/qscicommandset.h>
 #include "Preferences.h"
 #include "PlatformUtils.h"
 #include "settings.h"
+#include <ciso646> // C alternative tokens (xor)
+#include <boost/filesystem.hpp>
+namespace fs=boost::filesystem;
 
 class SettingsConverter {
 public:
@@ -19,10 +21,10 @@ public:
 
 QsciScintilla::WrapMode SettingsConverter::toWrapMode(Value val)
 {
-	std::string v = val.toString();
-	if (v == "Char") {
+	auto strVal = val.toString();
+	if (strVal == "Char") {
 		return QsciScintilla::WrapCharacter;
-	} else if (v == "Word") {
+	} else if (strVal == "Word") {
 		return QsciScintilla::WrapWord;
 	} else {
 		return QsciScintilla::WrapNone;
@@ -31,13 +33,13 @@ QsciScintilla::WrapMode SettingsConverter::toWrapMode(Value val)
 
 QsciScintilla::WrapVisualFlag SettingsConverter::toLineWrapVisualization(Value val)
 {
-	std::string v = val.toString();
-	if (v == "Text") {
+	auto strVal = val.toString();
+	if (strVal == "Text") {
 		return QsciScintilla::WrapFlagByText;
-	} else if (v == "Border") {
+	} else if (strVal == "Border") {
 		return QsciScintilla::WrapFlagByBorder;
 #if QSCINTILLA_VERSION >= 0x020700
-	} else if (v == "Margin") {
+	} else if (strVal == "Margin") {
 		return QsciScintilla::WrapFlagInMargin;
 #endif
 	} else {
@@ -47,10 +49,10 @@ QsciScintilla::WrapVisualFlag SettingsConverter::toLineWrapVisualization(Value v
 
 QsciScintilla::WrapIndentMode SettingsConverter::toLineWrapIndentationStyle(Value val)
 {
-	std::string v = val.toString();
-	if (v == "Same") {
+	auto strVal = val.toString();
+	if (strVal == "Same") {
 		return QsciScintilla::WrapIndentSame;
-	} else if (v == "Indented") {
+	} else if (strVal == "Indented") {
 		return QsciScintilla::WrapIndentIndented;
 	} else {
 		return QsciScintilla::WrapIndentFixed;
@@ -59,10 +61,10 @@ QsciScintilla::WrapIndentMode SettingsConverter::toLineWrapIndentationStyle(Valu
 
 QsciScintilla::WhitespaceVisibility SettingsConverter::toShowWhitespaces(Value val)
 {
-	std::string v = val.toString();
-	if (v == "Always") {
+	auto strVal = val.toString();
+	if (strVal == "Always") {
 		return QsciScintilla::WsVisible;
-	} else if (v == "AfterIndentation") {
+	} else if (strVal == "AfterIndentation") {
 		return QsciScintilla::WsVisibleAfterIndent;
 	} else {
 		return QsciScintilla::WsInvisible;
@@ -72,11 +74,11 @@ QsciScintilla::WhitespaceVisibility SettingsConverter::toShowWhitespaces(Value v
 EditorColorScheme::EditorColorScheme(fs::path path) : path(path)
 {
 	try {
-		boost::property_tree::read_json(boosty::stringy(path).c_str(), pt);
-		_name = QString(pt.get<std::string>("name").c_str());
+		boost::property_tree::read_json(path.generic_string(), pt);
+		_name = QString::fromStdString(pt.get<std::string>("name"));
 		_index = pt.get<int>("index");
 	} catch (const std::exception & e) {
-		PRINTB("Error reading color scheme file '%s': %s", boosty::stringy(path).c_str() % e.what());
+		PRINTB("Error reading color scheme file '%s': %s", path.generic_string() % e.what());
 		_name = "";
 		_index = 0;
 	}
@@ -134,21 +136,24 @@ ScintillaEditor::ScintillaEditor(QWidget *parent) : EditorInterface(parent)
 	// Ctrl-Shift-Z should redo on all platforms
 	c = qsci->standardCommands()->find(QsciCommand::Redo);
 	c->setKey(Qt::Key_Z | Qt::CTRL | Qt::SHIFT);
+	c->setAlternateKey(Qt::Key_Y | Qt::CTRL);
 
 	scintillaLayout->setContentsMargins(0, 0, 0, 0);
 	scintillaLayout->addWidget(qsci);
 
-	qsci->indicatorDefine(QsciScintilla::RoundBoxIndicator, indicatorNumber);
+	qsci->indicatorDefine(QsciScintilla::RoundBoxIndicator, errorIndicatorNumber);
+	qsci->indicatorDefine(QsciScintilla::RoundBoxIndicator , findIndicatorNumber); 
 	qsci->markerDefine(QsciScintilla::Circle, markerNumber);
 	qsci->setUtf8(true);
 	qsci->setFolding(QsciScintilla::BoxedTreeFoldStyle, 4);
 
-	lexer = new ScadLexer(this);
-	qsci->setLexer(lexer);
+	this->lexer = new ScadLexer(this);
+	qsci->setLexer(this->lexer);
 	initMargin();
 
 	connect(qsci, SIGNAL(textChanged()), this, SIGNAL(contentsChanged()));
 	connect(qsci, SIGNAL(modificationChanged(bool)), this, SIGNAL(modificationChanged(bool)));
+	qsci->installEventFilter(this);
 }
 
 /**
@@ -158,7 +163,7 @@ ScintillaEditor::ScintillaEditor(QWidget *parent) : EditorInterface(parent)
 void ScintillaEditor::applySettings()
 {
 	SettingsConverter conv;
-	Settings::Settings *s = Settings::Settings::inst();
+	auto s = Settings::Settings::inst();
 
 	qsci->setIndentationWidth(s->get(Settings::Settings::indentationWidth).toDouble());
 	qsci->setTabWidth(s->get(Settings::Settings::tabWidth).toDouble());
@@ -170,14 +175,20 @@ void ScintillaEditor::applySettings()
 	qsci->setWhitespaceVisibility(conv.toShowWhitespaces(s->get(Settings::Settings::showWhitespace)));
 	qsci->setWhitespaceSize(s->get(Settings::Settings::showWhitespaceSize).toDouble());
 	qsci->setAutoIndent(s->get(Settings::Settings::autoIndent).toBool());
+	qsci->setBackspaceUnindents(s->get(Settings::Settings::backspaceUnindents).toBool());
 
-	std::string indentStyle = s->get(Settings::Settings::indentStyle).toString();
+	auto indentStyle = s->get(Settings::Settings::indentStyle).toString();
 	qsci->setIndentationsUseTabs(indentStyle == "Tabs");
-	std::string tabKeyFunction = s->get(Settings::Settings::tabKeyFunction).toString();
+	auto tabKeyFunction = s->get(Settings::Settings::tabKeyFunction).toString();
 	qsci->setTabIndents(tabKeyFunction == "Indent");
 
 	qsci->setBraceMatching(s->get(Settings::Settings::enableBraceMatching).toBool() ? QsciScintilla::SloppyBraceMatch : QsciScintilla::NoBraceMatch);
 	qsci->setCaretLineVisible(s->get(Settings::Settings::highlightCurrentLine).toBool());
+    auto value = s->get(Settings::Settings::enableLineNumbers).toBool();
+    qsci->setMarginLineNumbers(1,value);
+
+    if (!value) qsci->setMarginWidth(1,20);
+    else qsci->setMarginWidth(1,QString(trunc(log10(qsci->lines())+4), '0'));
 }
 
 void ScintillaEditor::setPlainText(const QString &text)
@@ -209,34 +220,34 @@ void ScintillaEditor::highlightError(int error_pos)
 {
 	int line, index;
 	qsci->lineIndexFromPosition(error_pos, &line, &index);
-	qsci->fillIndicatorRange(line, index, line, index + 1, indicatorNumber);
+	qsci->fillIndicatorRange(line, index, line, index + 1, errorIndicatorNumber);
 	qsci->markerAdd(line, markerNumber);
 }
 
 void ScintillaEditor::unhighlightLastError()
 {
-	int totalLength = qsci->text().length();
+	auto totalLength = qsci->text().length();
 	int line, index;
 	qsci->lineIndexFromPosition(totalLength, &line, &index);
-	qsci->clearIndicatorRange(0, 0, line, index, indicatorNumber);
+	qsci->clearIndicatorRange(0, 0, line, index, errorIndicatorNumber);
 	qsci->markerDeleteAll(markerNumber);
 }
 
 QColor ScintillaEditor::readColor(const boost::property_tree::ptree &pt, const std::string name, const QColor defaultColor)
 {
 	try {
-		const std::string val = pt.get<std::string>(name);
+		const auto val = pt.get<std::string>(name);
 #if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 		if ((val.length() == 9) && (val.at(0) == '#')) {
-			const std::string rgb = std::string("#") + val.substr(3);
+			const auto rgb = std::string("#") + val.substr(3);
 			QColor qcol(rgb.c_str());
-			unsigned long alpha = std::strtoul(val.substr(1, 2).c_str(), 0, 16);
+			auto alpha = std::strtoul(val.substr(1, 2).c_str(), 0, 16);
 			qcol.setAlpha(alpha);
 			return qcol;
 		}
 #endif
 		return QColor(val.c_str());
-	} catch (std::exception e) {
+	} catch (const std::exception &e) {
 		return defaultColor;
 	}
 }
@@ -244,9 +255,9 @@ QColor ScintillaEditor::readColor(const boost::property_tree::ptree &pt, const s
 std::string ScintillaEditor::readString(const boost::property_tree::ptree &pt, const std::string name, const std::string defaultValue)
 {
 	try {
-		const std::string val = pt.get<std::string>(name);
+		const auto val = pt.get<std::string>(name);
 		return val;
-	} catch (std::exception e) {
+	} catch (const std::exception &e) {
 		return defaultValue;
 	}
 }
@@ -254,68 +265,70 @@ std::string ScintillaEditor::readString(const boost::property_tree::ptree &pt, c
 int ScintillaEditor::readInt(const boost::property_tree::ptree &pt, const std::string name, const int defaultValue)
 {
 	try {
-		const int val = pt.get<int>(name);
+		const auto val = pt.get<int>(name);
 		return val;
-	} catch (std::exception e) {
+	} catch (const std::exception &e) {
 		return defaultValue;
 	}
 }
 
 void ScintillaEditor::setColormap(const EditorColorScheme *colorScheme)
 {
-	const boost::property_tree::ptree & pt = colorScheme->propertyTree();
+	const auto & pt = colorScheme->propertyTree();
 
 	try {
-          QFont font = lexer->font(lexer->defaultStyle());
+          auto font = this->lexer->font(this->lexer->defaultStyle());
 		const QColor textColor(pt.get<std::string>("text").c_str());
 		const QColor paperColor(pt.get<std::string>("paper").c_str());
 
-		ScadLexer *l = new ScadLexer(this);
+		auto *newLexer = new ScadLexer(this);
 
 		// Keywords must be set before the lexer is attached to QScintilla
 		// as they seem to be read and cached at attach time.
 		boost::optional<const boost::property_tree::ptree&> keywords = pt.get_child_optional("keywords");
 		if (keywords.is_initialized()) {
-			l->setKeywords(1, readString(keywords.get(), "keyword-set1", ""));
-			l->setKeywords(2, readString(keywords.get(), "keyword-set2", ""));
-			l->setKeywords(3, readString(keywords.get(), "keyword-set-doc", ""));
-			l->setKeywords(4, readString(keywords.get(), "keyword-set3", ""));
+			newLexer->setKeywords(1, readString(keywords.get(), "keyword-set1", ""));
+			newLexer->setKeywords(2, readString(keywords.get(), "keyword-set2", ""));
+			newLexer->setKeywords(3, readString(keywords.get(), "keyword-set-doc", ""));
+			newLexer->setKeywords(4, readString(keywords.get(), "keyword-set3", ""));
 		}
 
-		qsci->setLexer(l);
-		delete lexer;
-		lexer = l;
+		qsci->setLexer(newLexer);
+		delete this->lexer;
+		this->lexer = newLexer;
 
 		// All other properties must be set after attaching to QSCintilla so
 		// the editor gets the change events and updates itself to match
-		l->setFont(font);
-		l->setColor(textColor);
-		l->setPaper(paperColor);
+		newLexer->setFont(font);
+		newLexer->setColor(textColor);
+		newLexer->setPaper(paperColor);
                 // Somehow, the margin font got lost when we deleted the old lexer
                 qsci->setMarginsFont(font);
 
-		const boost::property_tree::ptree& colors = pt.get_child("colors");
-		l->setColor(readColor(colors, "keyword1", textColor), QsciLexerCPP::Keyword);
-		l->setColor(readColor(colors, "keyword2", textColor), QsciLexerCPP::KeywordSet2);
-		l->setColor(readColor(colors, "keyword3", textColor), QsciLexerCPP::GlobalClass);
-		l->setColor(readColor(colors, "number", textColor), QsciLexerCPP::Number);
-		l->setColor(readColor(colors, "string", textColor), QsciLexerCPP::SingleQuotedString);
-		l->setColor(readColor(colors, "string", textColor), QsciLexerCPP::DoubleQuotedString);
-		l->setColor(readColor(colors, "operator", textColor), QsciLexerCPP::Operator);
-		l->setColor(readColor(colors, "comment", textColor), QsciLexerCPP::Comment);
-		l->setColor(readColor(colors, "commentline", textColor), QsciLexerCPP::CommentLine);
-		l->setColor(readColor(colors, "commentdoc", textColor), QsciLexerCPP::CommentDoc);
-		l->setColor(readColor(colors, "commentdoc", textColor), QsciLexerCPP::CommentLineDoc);
-		l->setColor(readColor(colors, "commentdockeyword", textColor), QsciLexerCPP::CommentDocKeyword);
+		const auto& colors = pt.get_child("colors");
+		newLexer->setColor(readColor(colors, "keyword1", textColor), QsciLexerCPP::Keyword);
+		newLexer->setColor(readColor(colors, "keyword2", textColor), QsciLexerCPP::KeywordSet2);
+		newLexer->setColor(readColor(colors, "keyword3", textColor), QsciLexerCPP::GlobalClass);
+		newLexer->setColor(readColor(colors, "number", textColor), QsciLexerCPP::Number);
+		//newLexer->setColor(readColor(colors, "string", textColor), QsciLexerCPP::SingleQuotedString); //currently, we do not support SingleQuotedStrings
+		newLexer->setColor(readColor(colors, "string", textColor), QsciLexerCPP::DoubleQuotedString);
+		newLexer->setColor(readColor(colors, "operator", textColor), QsciLexerCPP::Operator);
+		newLexer->setColor(readColor(colors, "comment", textColor), QsciLexerCPP::Comment);
+		newLexer->setColor(readColor(colors, "commentline", textColor), QsciLexerCPP::CommentLine);
+		newLexer->setColor(readColor(colors, "commentdoc", textColor), QsciLexerCPP::CommentDoc);
+		newLexer->setColor(readColor(colors, "commentdoc", textColor), QsciLexerCPP::CommentLineDoc);
+		newLexer->setColor(readColor(colors, "commentdockeyword", textColor), QsciLexerCPP::CommentDocKeyword);
 
-		const boost::property_tree::ptree& caret = pt.get_child("caret");
+		const auto& caret = pt.get_child("caret");
 		qsci->setCaretWidth(readInt(caret, "width", 1));
 		qsci->setCaretForegroundColor(readColor(caret, "foreground", textColor));
 		qsci->setCaretLineBackgroundColor(readColor(caret, "line-background", paperColor));
 
 		qsci->setMarkerBackgroundColor(readColor(colors, "error-marker", QColor(255, 0, 0, 100)), markerNumber);
-		qsci->setIndicatorForegroundColor(readColor(colors, "error-indicator", QColor(255, 0, 0, 100)), indicatorNumber);
-		qsci->setIndicatorOutlineColor(readColor(colors, "error-indicator-outline", QColor(255, 0, 0, 100)), indicatorNumber);
+        qsci->setIndicatorForegroundColor(readColor(colors, "error-indicator", QColor(255, 0, 0, 100)), errorIndicatorNumber);//red
+        qsci->setIndicatorOutlineColor(readColor(colors, "error-indicator-outline", QColor(255, 0, 0, 100)), errorIndicatorNumber);//red
+        qsci->setIndicatorForegroundColor(readColor(colors, "find-indicator", QColor(255, 255, 0, 100)), findIndicatorNumber);//yellow
+        qsci->setIndicatorOutlineColor(readColor(colors, "find-indicator-outline", QColor(255, 255, 0, 100)), findIndicatorNumber);//yellow
 		qsci->setWhitespaceForegroundColor(readColor(colors, "whitespace-foreground", textColor));
 		qsci->setMarginsBackgroundColor(readColor(colors, "margin-background", paperColor));
 		qsci->setMarginsForegroundColor(readColor(colors, "margin-foreground", textColor));
@@ -328,20 +341,22 @@ void ScintillaEditor::setColormap(const EditorColorScheme *colorScheme)
 		qsci->setSelectionForegroundColor(readColor(colors, "selection-foreground", paperColor));
 		qsci->setSelectionBackgroundColor(readColor(colors, "selection-background", textColor));
 		qsci->setEdgeColor(readColor(colors, "edge", textColor));
-	} catch (std::exception e) {
+	} catch (const std::exception &e) {
 		noColor();
 	}
 }
 
 void ScintillaEditor::noColor()
 {
-	lexer->setPaper(Qt::white);
-	lexer->setColor(Qt::black);
+	this->lexer->setPaper(Qt::white);
+	this->lexer->setColor(Qt::black);
 	qsci->setCaretWidth(2);
 	qsci->setCaretForegroundColor(Qt::black);
 	qsci->setMarkerBackgroundColor(QColor(255, 0, 0, 100), markerNumber);
-	qsci->setIndicatorForegroundColor(QColor(255, 0, 0, 128), indicatorNumber);
-	qsci->setIndicatorOutlineColor(QColor(0, 0, 0, 255), indicatorNumber); // only alpha part is used
+    qsci->setIndicatorForegroundColor(QColor(255, 0, 0, 128), errorIndicatorNumber);//red
+    qsci->setIndicatorOutlineColor(QColor(0, 0, 0, 255), errorIndicatorNumber); // only alpha part is used
+    qsci->setIndicatorForegroundColor(QColor(255, 255, 0, 128), findIndicatorNumber);//yellow
+    qsci->setIndicatorOutlineColor(QColor(0, 0, 0, 255), findIndicatorNumber); // only alpha part is used
 	qsci->setCaretLineBackgroundColor(Qt::white);
 	qsci->setWhitespaceForegroundColor(Qt::black);
 	qsci->setSelectionForegroundColor(Qt::black);
@@ -358,26 +373,18 @@ void ScintillaEditor::noColor()
 
 void ScintillaEditor::enumerateColorSchemesInPath(ScintillaEditor::colorscheme_set_t &result_set, const fs::path path)
 {
-	const fs::path color_schemes = path / "color-schemes" / "editor";
-
-	fs::directory_iterator end_iter;
+	const auto color_schemes = path / "color-schemes" / "editor";
 
 	if (fs::exists(color_schemes) && fs::is_directory(color_schemes)) {
-		for (fs::directory_iterator dir_iter(color_schemes); dir_iter != end_iter; ++dir_iter) {
-			if (!fs::is_regular_file(dir_iter->status())) {
-				continue;
-			}
+		for (const auto& dirEntry : boost::make_iterator_range(fs::directory_iterator{color_schemes}, {})) {
+			if (!fs::is_regular_file(dirEntry.status())) continue;
 
-			const fs::path path = (*dir_iter).path();
-			if (!(path.extension() == ".json")) {
-				continue;
-			}
+			const auto &path = dirEntry.path();
+			if (!(path.extension() == ".json")) continue;
 
-			EditorColorScheme *colorScheme = new EditorColorScheme(path);
+			auto colorScheme = std::make_shared<EditorColorScheme>(path);
 			if (colorScheme->valid()) {
-				result_set.insert(colorscheme_set_t::value_type(colorScheme->index(), boost::shared_ptr<EditorColorScheme>(colorScheme)));
-			} else {
-				delete colorScheme;
+				result_set.emplace(colorScheme->index(), colorScheme);
 			}
 		}
 	}
@@ -395,23 +402,24 @@ ScintillaEditor::colorscheme_set_t ScintillaEditor::enumerateColorSchemes()
 
 QStringList ScintillaEditor::colorSchemes()
 {
-	const colorscheme_set_t colorscheme_set = enumerateColorSchemes();
-
 	QStringList colorSchemes;
-	for (colorscheme_set_t::const_iterator it = colorscheme_set.begin(); it != colorscheme_set.end(); it++) {
-		colorSchemes << (*it).second.get()->name();
+	for (const auto &colorSchemeEntry : enumerateColorSchemes()) {
+		colorSchemes << colorSchemeEntry.second.get()->name();
 	}
 	colorSchemes << "Off";
 
 	return colorSchemes;
 }
 
+bool ScintillaEditor::canUndo()
+{
+    return qsci->isUndoAvailable();
+}
+
 void ScintillaEditor::setHighlightScheme(const QString &name)
 {
-	const colorscheme_set_t colorscheme_set = enumerateColorSchemes();
-
-	for (colorscheme_set_t::const_iterator it = colorscheme_set.begin(); it != colorscheme_set.end(); it++) {
-		const EditorColorScheme *colorScheme = (*it).second.get();
+	for (const auto &colorSchemeEntry : enumerateColorSchemes()) {
+		const auto colorScheme = colorSchemeEntry.second.get();
 		if (colorScheme->name() == name) {
 			setColormap(colorScheme);
 			return;
@@ -471,7 +479,7 @@ void ScintillaEditor::initFont(const QString& fontName, uint size)
 {
   this->currentFont = QFont(fontName, size);
   this->currentFont.setFixedPitch(true);
-  lexer->setFont(this->currentFont);
+  this->lexer->setFont(this->currentFont);
   qsci->setMarginsFont(this->currentFont);
   onTextChanged(); // Update margin width
 }
@@ -484,8 +492,34 @@ void ScintillaEditor::initMargin()
 
 void ScintillaEditor::onTextChanged()
 {
+  auto s = Settings::Settings::inst();
   QFontMetrics fontmetrics(this->currentFont);
-  qsci->setMarginWidth(1, QString(trunc(log10(qsci->lines())+2), '0'));
+  auto enableLineNumbers = s->get(Settings::Settings::enableLineNumbers).toBool();
+
+  if (!enableLineNumbers) qsci->setMarginWidth(1, 20);
+  else qsci->setMarginWidth(1, QString(trunc(log10(qsci->lines()) + 4), '0'));
+}
+
+int ScintillaEditor::updateFindIndicators(const QString &findText, bool visibility)
+{
+	int findwordcount{0};
+
+    auto txt = qsci->text();
+
+    qsci->SendScintilla(qsci->SCI_INDICATORCLEARRANGE, 0, txt.length());
+
+    auto pos = txt.indexOf(findText);
+    auto len = findText.length();
+    if (visibility && len>0) {
+        while (pos!=-1){
+            findwordcount++;
+            qsci->SendScintilla(qsci->SCI_SETINDICATORCURRENT, findIndicatorNumber);
+            qsci->SendScintilla(qsci->SCI_INDICATORFILLRANGE, pos, len);
+            pos = txt.indexOf(findText,pos+len);
+        };
+    }
+    //qsci->findFirst(findText, false, false, false, true, true, savelineFrom, saveindexFrom);
+    return findwordcount;
 }
 
 bool ScintillaEditor::find(const QString &expr, bool findNext, bool findBackwards)
@@ -507,7 +541,7 @@ bool ScintillaEditor::find(const QString &expr, bool findNext, bool findBackward
 
 void ScintillaEditor::replaceSelectedText(const QString &newText)
 {
-	if (qsci->selectedText() != newText) qsci->replaceSelectedText(newText);
+    if ((qsci->selectedText() != newText)&&(qsci->hasSelectedText())) qsci->replaceSelectedText(newText);
 }
 
 void ScintillaEditor::replaceAll(const QString &findText, const QString &replaceText)
@@ -515,7 +549,12 @@ void ScintillaEditor::replaceAll(const QString &findText, const QString &replace
   // We need to issue a Select All first due to a bug in QScintilla:
   // It doesn't update the find range when just doing findFirst() + findNext() causing the search
   // to end prematurely if the replaced string is larger than the selected string.
-#if QSCINTILLA_VERSION >= 0x020700
+#if QSCINTILLA_VERSION >= 0x020903
+  // QScintilla bug seams to be fixed in 2.9.3
+  if (qsci->findFirst(findText,
+                      false /*re*/, false /*cs*/, false /*wo*/,
+                      false /*wrap*/, true /*forward*/, 0, 0)) {
+#elif QSCINTILLA_VERSION >= 0x020700
   qsci->selectAll();
   if (qsci->findFirstInSelection(findText, 
                       false /*re*/, false /*cs*/, false /*wo*/, 
@@ -533,6 +572,7 @@ void ScintillaEditor::replaceAll(const QString &findText, const QString &replace
   }
 }
 
+
 void ScintillaEditor::getRange(int *lineFrom, int *lineTo)
 {
 	int indexFrom, indexTo;
@@ -567,7 +607,7 @@ void ScintillaEditor::unindentSelection()
 
 void ScintillaEditor::commentSelection()
 {
-	bool hasSelection = qsci->hasSelectedText();
+	auto hasSelection = qsci->hasSelectedText();
 
 	int lineFrom, lineTo;
 	getRange(&lineFrom, &lineTo);
@@ -582,7 +622,7 @@ void ScintillaEditor::commentSelection()
 
 void ScintillaEditor::uncommentSelection()
 {
-	bool hasSelection = qsci->hasSelectedText();
+	auto hasSelection = qsci->hasSelectedText();
 
 	int lineFrom, lineTo;
 	getRange(&lineFrom, &lineTo);
@@ -602,3 +642,148 @@ QString ScintillaEditor::selectedText()
 {
 	return qsci->selectedText();
 }
+
+bool ScintillaEditor::eventFilter(QObject* obj, QEvent *e)
+{
+	static bool wasChanged=false;
+	static bool previewAfterUndo=false;
+
+	if (obj != qsci) return EditorInterface::eventFilter(obj, e);
+
+	if (e->type()==QEvent::KeyPress || e->type()==QEvent::KeyRelease) {
+		auto *ke = static_cast<QKeyEvent*>(e);
+#ifdef Q_OS_MAC
+                int navigateOnNumberModifiers = Qt::AltModifier | Qt::ShiftModifier | Qt :: KeypadModifier;
+#else
+                int navigateOnNumberModifiers = Qt::AltModifier;
+#endif
+		if (ke->modifiers()  == navigateOnNumberModifiers) {
+			switch (ke->key())
+			{
+				case Qt::Key_Left:
+				case Qt::Key_Right:
+					if (e->type()==QEvent::KeyPress) {
+						navigateOnNumber(ke->key());
+					}
+					return true;
+
+				case Qt::Key_Up:
+				case Qt::Key_Down:
+					if (e->type()==QEvent::KeyPress) {
+						if (!wasChanged) qsci->beginUndoAction();
+						if (modifyNumber(ke->key())) {
+							wasChanged=true;
+							previewAfterUndo=true;
+						}
+						if (!wasChanged) qsci->endUndoAction();
+					}
+					return true;
+			}
+		}
+		if (previewAfterUndo && e->type()==QEvent::KeyPress) {
+			int k=ke->key() | ke->modifiers();
+			if (wasChanged) qsci->endUndoAction();
+			wasChanged=false;
+			auto *cmd=qsci->standardCommands()->boundTo(k);
+			if ( cmd && ( cmd->command()==QsciCommand::Undo || cmd->command()==QsciCommand::Redo ) )
+				QTimer::singleShot(0,this,SIGNAL(previewRequest()));
+			else if ( cmd || !ke->text().isEmpty() ) {
+				// any insert or command (but not undo/redo) cancels the preview after undo
+				previewAfterUndo=false;
+			}
+		}
+	}
+	return false;
+}
+
+
+void ScintillaEditor::navigateOnNumber(int key)
+{
+	int line, index;
+	qsci->getCursorPosition(&line, &index);
+	auto text=qsci->text(line);
+	auto left=text.left(index);
+	auto dotOnLeft=left.contains(QRegExp("\\.\\d*$"));
+	auto dotJustLeft=index>1 && text[index-2]=='.';
+	auto dotJustRight=text[index]=='.';
+	auto numOnLeft=left.contains(QRegExp("\\d\\.?$")) || left.endsWith("-.");
+	auto numOnRight=text.indexOf(QRegExp("\\.?\\d"),index)==index;
+
+	switch (key)
+	{
+		case Qt::Key_Left:
+			if (numOnLeft)
+				qsci->setCursorPosition(line, index-(dotJustLeft?2:1));
+			break;
+
+		case Qt::Key_Right:
+			if (numOnRight)
+				qsci->setCursorPosition(line, index+(dotJustRight?2:1));
+			else if (numOnLeft) {
+				// add trailing zero
+				if (!dotOnLeft) {
+					qsci->insert(".0");
+					index++;
+				} else {
+					qsci->insert("0");
+				}
+				qsci->setCursorPosition(line, index+1);
+			}
+			break;
+	}
+}
+
+bool ScintillaEditor::modifyNumber(int key)
+{
+	int line, index;
+	qsci->getCursorPosition(&line, &index);
+	auto text=qsci->text(line);
+
+	int lineFrom, indexFrom, lineTo, indexTo;
+	qsci->getSelection(&lineFrom, &indexFrom, &lineTo, &indexTo);
+	auto hadSelection=qsci->hasSelectedText();
+
+	auto begin=QRegExp("[-+]?\\d*\\.?\\d*$").indexIn(text.left(index));
+	auto end=text.indexOf(QRegExp("[^0-9.]"),index);
+	if (end<0) end=text.length();
+	auto nr=text.mid(begin,end-begin);
+	if ( !(nr.contains(QRegExp("^[-+]?\\d*\\.?\\d*$")) && nr.contains(QRegExp("\\d"))) ) return false;
+	auto sign=nr[0]=='+'||nr[0]=='-';
+	if (nr.endsWith('.')) nr=nr.left(nr.length()-1);
+	auto curpos=index-begin;
+	auto dotpos=nr.indexOf('.');
+	auto decimals=dotpos<0?0:nr.length()-dotpos-1;
+	auto number=(dotpos<0)?nr.toLongLong():(nr.left(dotpos)+nr.mid(dotpos+1)).toLongLong();
+	auto tail=nr.length()-curpos;
+	auto exponent=tail-((dotpos>=curpos)?1:0);
+	long long int step=1;
+	for (int i=exponent; i>0; i--) step*=10;
+
+	switch (key) {
+		case Qt::Key_Up:   number+=step; break;
+		case Qt::Key_Down: number-=step; break;
+	}
+	auto negative=number<0;
+	if (negative) number=-number;
+	auto newnr=QString::number(number);
+	if (decimals) {
+		if (newnr.length()<=decimals) newnr.prepend(QString(decimals-newnr.length()+1,'0'));
+		newnr=newnr.left(newnr.length()-decimals)+"."+newnr.right(decimals);
+	}
+	if (tail>newnr.length()) {
+		newnr.prepend(QString(tail-newnr.length(),'0'));
+	}
+	if (negative) newnr.prepend('-');
+	else if (sign) newnr.prepend('+');
+	qsci->setSelection(line, begin, line, end);
+	qsci->replaceSelectedText(newnr);
+
+	qsci->selectAll(false);
+	if (hadSelection)
+	{
+		qsci->setSelection(lineFrom, indexFrom, lineTo, indexTo);
+	}
+	qsci->setCursorPosition(line, begin+newnr.length()-tail);
+	emit previewRequest();
+	return true;
+}
