@@ -9,10 +9,15 @@
 #ifndef Q_MOC_RUN
 #include <boost/variant.hpp>
 #include <boost/lexical_cast.hpp>
+#include <glib.h>
+
 #endif
-#include <boost/cstdint.hpp>
+#include <cstdint>
 #include "memory.h"
 
+class tostring_visitor;
+class tostream_visitor;
+
 class QuotedString : public std::string
 {
 public:
@@ -39,7 +44,7 @@ private:
 	void normalize();
 	
 public:
-	typedef enum { RANGE_TYPE_BEGIN, RANGE_TYPE_RUNNING, RANGE_TYPE_END } type_t;
+	enum class type_t { RANGE_TYPE_BEGIN, RANGE_TYPE_RUNNING, RANGE_TYPE_END };
   
 	class iterator {
 	public:
@@ -84,21 +89,22 @@ public:
 	double step_value() { return step_val; }
 	double end_value() { return end_val; }
 	
-	iterator begin() { return iterator(*this, RANGE_TYPE_BEGIN); }
-	iterator end() { return iterator(*this, RANGE_TYPE_END); }
+	iterator begin() { return iterator(*this, type_t::RANGE_TYPE_BEGIN); }
+	iterator end() { return iterator(*this, type_t::RANGE_TYPE_END); }
 	
 	/// return number of values, max uint32_t value if step is 0 or range is infinite
-	boost::uint32_t numValues() const;
+	uint32_t numValues() const;
   
 	friend class chr_visitor;
 	friend class tostring_visitor;
+	friend class tostream_visitor;
 	friend class bracket_visitor;
 };
 
 class ValuePtr : public shared_ptr<const class Value>
 {
 public:
-  static ValuePtr undefined;
+  static const ValuePtr undefined;
 
 	ValuePtr();
 	explicit ValuePtr(const Value &v);
@@ -133,12 +139,32 @@ public:
 private:
 };
 
+
+class str_utf8_wrapper : public std::string
+{
+public:
+	str_utf8_wrapper() : std::string(), cached_len(-1) { }
+	str_utf8_wrapper( const std::string& s ) : std::string( s ), cached_len(-1) { }
+	str_utf8_wrapper( size_t n, char c ) : std::string(n, c), cached_len(-1) { }
+	~str_utf8_wrapper() {}
+	
+	glong get_utf8_strlen() const {
+		if (cached_len < 0) {
+			cached_len = g_utf8_strlen(this->c_str(), this->size());
+		}
+		return cached_len;
+	};
+private:
+	mutable glong cached_len;
+};
+
+
 class Value
 {
 public:
 	typedef std::vector<ValuePtr> VectorType;
 
-  enum ValueType {
+  enum class ValueType {
     UNDEFINED,
     BOOL,
     NUMBER,
@@ -146,7 +172,7 @@ public:
     VECTOR,
     RANGE
   };
-  static Value undefined;
+  static const Value undefined;
 
   Value();
   Value(bool v);
@@ -169,10 +195,16 @@ public:
   bool getFiniteDouble(double &v) const;
   bool toBool() const;
   std::string toString() const;
+  std::string toString(const tostring_visitor *visitor) const;
+  std::string toEchoString() const;
+  std::string toEchoString(const tostring_visitor *visitor) const;
+  void toStream(std::ostringstream &stream) const;
+  void toStream(const tostream_visitor *visitor) const;
   std::string chrString() const;
   const VectorType &toVector() const;
   bool getVec2(double &x, double &y, bool ignoreInfinite = false) const;
-  bool getVec3(double &x, double &y, double &z, double defaultval = 0.0) const;
+  bool getVec3(double &x, double &y, double &z) const;
+  bool getVec3(double &x, double &y, double &z, double defaultval) const;
   RangeType toRange() const;
 
 	operator bool() const { return this->toBool(); }
@@ -193,12 +225,12 @@ public:
   Value operator%(const Value &v) const;
 
   friend std::ostream &operator<<(std::ostream &stream, const Value &value) {
-    if (value.type() == Value::STRING) stream << QuotedString(value.toString());
+    if (value.type() == Value::ValueType::STRING) stream << QuotedString(value.toString());
     else stream << value.toString();
     return stream;
   }
 
-  typedef boost::variant< boost::blank, bool, double, std::string, VectorType, RangeType > Variant;
+  typedef boost::variant< boost::blank, bool, double, str_utf8_wrapper, VectorType, RangeType > Variant;
 
 private:
   static Value multvecnum(const Value &vecval, const Value &numval);
@@ -208,3 +240,4 @@ private:
   Variant value;
 };
 
+void utf8_split(const std::string& str, std::function<void(ValuePtr)> f);
