@@ -37,7 +37,7 @@ a time, to avoid confusion.
 
 
 #include <Eigen/Core>
-#if not EIGEN_VERSION_AT_LEAST( 3,0,0 )
+#if !EIGEN_VERSION_AT_LEAST( 3,0,0 )
 #error eigen library missing or version too old. See README.md. To force compile, run qmake CONFIG+=skip-version-check
 #else
 
@@ -50,7 +50,10 @@ a time, to avoid confusion.
 
 
 #ifdef ENABLE_CGAL
+#pragma push_macro("NDEBUG")
+#undef NDEBUG
 #include <CGAL/version.h>
+#pragma pop_macro("NDEBUG")
 
 #if CGAL_VERSION_NR < 1030601000
 #error CGAL library missing or version too old. See README.md. To force compile, run qmake CONFIG+=skip-version-check
@@ -76,7 +79,7 @@ a time, to avoid confusion.
 
 #ifdef ENABLE_OPENCSG
 #include <GL/glew.h>
-// kludge - GLEW doesnt have compiler-accessible version numbering
+// kludge - GLEW doesn't have compiler-accessible version numbering
 #ifndef GLEW_ARB_occlusion_query2
 #error GLEW library missing or version too old. See README.md. To force compile, run qmake CONFIG+=skip-version-check
 #else
@@ -89,7 +92,7 @@ a time, to avoid confusion.
 #else
 #endif // ENABLE_OPENCSG
 
-#ifndef OPENSCAD_TESTING
+#ifndef OPENSCAD_NOGUI
 #include <QtCore/qglobal.h>
 #if QT_VERSION < 0x040400
 #error QT library missing or version too old. See README.md. To force compile, run qmake CONFIG+=skip-version-check
